@@ -268,64 +268,6 @@ int ctdb_ctrl_get_dbmap(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 	return 0;
 }
 
-int ctdb_ctrl_pull_db(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
-		      struct ctdb_client_context *client, int destnode,
-		      struct timeval timeout, struct ctdb_pulldb *pulldb,
-		      struct ctdb_rec_buffer **recbuf)
-{
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
-	int ret;
-
-	ctdb_req_control_pull_db(&request, pulldb);
-	ret = ctdb_client_control(mem_ctx, ev, client, destnode, timeout,
-				  &request, &reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control PULL_DB failed to node %u, ret=%d\n",
-		       destnode, ret));
-		return ret;
-	}
-
-	ret = ctdb_reply_control_pull_db(reply, mem_ctx, recbuf);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control PULL_DB failed, ret=%d\n", ret));
-		return ret;
-	}
-
-	return 0;
-}
-
-int ctdb_ctrl_push_db(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
-		      struct ctdb_client_context *client, int destnode,
-		      struct timeval timeout, struct ctdb_rec_buffer *recbuf)
-{
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
-	int ret;
-
-	ctdb_req_control_push_db(&request, recbuf);
-	ret = ctdb_client_control(mem_ctx, ev, client, destnode, timeout,
-				  &request, &reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control PUSH_DB failed to node %u, ret=%d\n",
-		       destnode, ret));
-		return ret;
-	}
-
-	ret = ctdb_reply_control_push_db(reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control PUSH_DB failed, ret=%d\n", ret));
-		return ret;
-	}
-
-	return 0;
-}
-
-
 int ctdb_ctrl_get_recmode(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 			  struct ctdb_client_context *client,
 			  int destnode, struct timeval timeout,
@@ -673,64 +615,6 @@ int ctdb_ctrl_get_pid(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 	return 0;
 }
 
-int ctdb_ctrl_get_recmaster(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
-			    struct ctdb_client_context *client,
-			    int destnode, struct timeval timeout,
-			    uint32_t *recmaster)
-{
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
-	int ret;
-
-	ctdb_req_control_get_recmaster(&request);
-	ret = ctdb_client_control(mem_ctx, ev, client, destnode, timeout,
-				  &request, &reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control GET_RECMASTER failed to node %u, ret=%d\n",
-		       destnode, ret));
-		return ret;
-	}
-
-	ret = ctdb_reply_control_get_recmaster(reply, recmaster);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control GET_RECMASTER failed, ret=%d\n", ret));
-		return ret;
-	}
-
-	return 0;
-}
-
-int ctdb_ctrl_set_recmaster(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
-			    struct ctdb_client_context *client,
-			    int destnode, struct timeval timeout,
-			    uint32_t recmaster)
-{
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
-	int ret;
-
-	ctdb_req_control_set_recmaster(&request, recmaster);
-	ret = ctdb_client_control(mem_ctx, ev, client, destnode, timeout,
-				  &request, &reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control SET_RECMASTER failed to node %u, ret=%d\n",
-		       destnode, ret));
-		return ret;
-	}
-
-	ret = ctdb_reply_control_set_recmaster(reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control SET_RECMASTER failed, ret=%d\n", ret));
-		return ret;
-	}
-
-	return 0;
-}
-
 int ctdb_ctrl_freeze(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 		     struct ctdb_client_context *client,
 		     int destnode, struct timeval timeout,
@@ -2226,34 +2110,6 @@ int ctdb_ctrl_reload_public_ips(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 	return 0;
 }
 
-int ctdb_ctrl_ipreallocated(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
-			    struct ctdb_client_context *client,
-			    int destnode, struct timeval timeout)
-{
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
-	int ret;
-
-	ctdb_req_control_ipreallocated(&request);
-	ret = ctdb_client_control(mem_ctx, ev, client, destnode, timeout,
-				  &request, &reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control IPREALLOCATED failed to node %u, ret=%d\n",
-		       destnode, ret));
-		return ret;
-	}
-
-	ret = ctdb_reply_control_ipreallocated(reply);
-	if (ret != 0) {
-		DEBUG(DEBUG_ERR,
-		      ("Control IPREALLOCATED failed, ret=%d\n", ret));
-		return ret;
-	}
-
-	return 0;
-}
-
 int ctdb_ctrl_get_runstate(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
 			   struct ctdb_client_context *client,
 			   int destnode, struct timeval timeout,
@@ -2725,8 +2581,10 @@ int ctdb_ctrl_disable_node(TALLOC_CTX *mem_ctx,
 			   int destnode,
 			   struct timeval timeout)
 {
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
+	struct ctdb_req_control request = {
+		.opcode = 0,
+	};
+	struct ctdb_reply_control *reply = NULL;
 	int ret;
 
 	ctdb_req_control_disable_node(&request);
@@ -2759,8 +2617,10 @@ int ctdb_ctrl_enable_node(TALLOC_CTX *mem_ctx,
 			  int destnode,
 			  struct timeval timeout)
 {
-	struct ctdb_req_control request;
-	struct ctdb_reply_control *reply;
+	struct ctdb_req_control request = {
+		.opcode = 0,
+	};
+	struct ctdb_reply_control *reply = NULL;
 	int ret;
 
 	ctdb_req_control_enable_node(&request);
