@@ -14,7 +14,7 @@ set_mem_usage 87 87
 required_result 1 <<EOF
 ERROR: System memory utilization 87% >= threshold 80%
 $FAKE_PROC_MEMINFO
-$(ps foobar)
+$(ps auxfww)
 EOF
 
 simple_test
