@@ -6,7 +6,9 @@ setup()
 
 	if [ "$1" != "down" ]; then
 		service "$_service_name" start
+		setup_tcp_listen 21
 	else
 		service "$_service_name" force-stopped
+		setup_tcp_listen ""
 	fi
 }
