@@ -22,10 +22,8 @@
 #include "dbwrap/dbwrap.h"
 #include "dbwrap/dbwrap_tdb.h"
 #include "tdb.h"
-#include "lib/param/param.h"
 
 struct db_context *dbwrap_local_open(TALLOC_CTX *mem_ctx,
-				     struct loadparm_context *lp_ctx,
 				     const char *name,
 				     int hash_size, int tdb_flags,
 				     int open_flags, mode_t mode,
@@ -34,9 +32,15 @@ struct db_context *dbwrap_local_open(TALLOC_CTX *mem_ctx,
 {
 	struct db_context *db = NULL;
 
-	db = db_open_tdb(mem_ctx, lp_ctx, name, hash_size,
-			 tdb_flags, open_flags, mode,
-			 lock_order, dbwrap_flags);
+	db = db_open_tdb(
+		mem_ctx,
+		name,
+		hash_size,
+		tdb_flags,
+		open_flags,
+		mode,
+		lock_order,
+		dbwrap_flags);
 
 	return db;
 }
