@@ -34,6 +34,7 @@
 #include "replace.h"
 #include "system/filesys.h"
 #include "system/wait.h"
+#include "lib/util/sys_rw.h"
 
 #ifdef HAVE_PTY_H
 #include <pty.h>
@@ -58,6 +59,9 @@
 #else
 const char progname[] = "unknown program";
 
+static void err(int eval, const char *fmt, ...) PRINTF_ATTRIBUTE(2, 0);
+static void errx(int eval, const char *fmt, ...) PRINTF_ATTRIBUTE(2, 0);
+
 static void err(int eval, const char *fmt, ...)
 {
 	int err_errno = errno;
@@ -176,82 +180,67 @@ static char *iscmd(const char *buf, const char *s)
 	return strdup(buf + len);
 }
 
-/*******************************************************************
-A write wrapper that will deal with EINTR.
-********************************************************************/
-
-static ssize_t sys_write(int fd, const void *buf, size_t count)
-{
-	ssize_t ret;
-
-	do {
-		ret = write(fd, buf, count);
-#if defined(EWOULDBLOCK)
-	} while (ret == -1 && (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK));
-#else
-	} while (ret == -1 && (errno == EINTR || errno == EAGAIN));
-#endif
-	return ret;
-}
-
 static void parse_configuration(const char *fn)
 {
 	struct command *c;
-	char s[1024];
+	char *conf_line = NULL;
 	char *str;
+	size_t len = 0;
 	unsigned int lineno = 0;
 	FILE *cmd;
+	const char *err_message = NULL;
+	char err_buf[256];
 
 	cmd = fopen(fn, "r");
 	if (cmd == NULL)
 		err(1, "open: %s", fn);
 
-	while (fgets(s, sizeof(s),  cmd) != NULL) {
-
-		s[strcspn(s, "#\n")] = '\0';
+	while (getline(&conf_line, &len, cmd) != -1) {
 		lineno++;
 
+		conf_line[strcspn(conf_line, "#\n")] = '\0';
+		if (conf_line[0] == '\0') {
+			continue;
+		}
+
 		c = calloc(1, sizeof(*c));
-		if (c == NULL)
-			errx(1, "malloc");
+		if (c == NULL) {
+			err_message = "calloc failed";
+			goto out;
+		}
 
 		c->lineno = lineno;
+
+		if ((str = iscmd(conf_line, "expect ")) != NULL) {
+			c->type = CMD_EXPECT;
+		} else if ((str = iscmd(conf_line, "send ")) != NULL) {
+			c->type = CMD_SEND;
+		} else if ((str = iscmd(conf_line, "password ")) != NULL) {
+			c->type = CMD_PASSWORD;
+		} else {
+			free(c);
+			snprintf(err_buf,
+				 sizeof(err_buf),
+				 "Invalid command on line %d: %s",
+				 lineno,
+				 conf_line);
+			err_message = err_buf;
+			goto out;
+		}
+
+		c->str = str;
+
 		(*next) = c;
 		next = &(c->next);
-
-		if ((str = iscmd(s, "expect ")) != NULL) {
-			c->type = CMD_EXPECT;
-			c->str = str;
-		} else if ((str = iscmd(s, "send ")) != NULL) {
-			c->type = CMD_SEND;
-			c->str = str;
-		} else if ((str = iscmd(s, "password ")) != NULL) {
-			c->type = CMD_PASSWORD;
-			c->str = str;
-		} else
-			errx(1, "Invalid command on line %d: %s", lineno, s);
 	}
-
+out:
+	free(conf_line);
 	fclose(cmd);
-}
-
-/* A wrapper to close als file descriptors above the given fd */
-static int sys_closefrom(int fd)
-{
-	int num = getdtablesize();
-
-	if (num < 0) {
-		num = 1024;
+	if (err_message) {
+		errx(1, "%s", err_message);
 	}
-
-	for (; fd <= num; fd++) {
-		close(fd);
-	}
-
-	return 0;
 }
 
-
 /*
  *
  */
@@ -401,7 +390,7 @@ int main(int argc, const char **argv)
 {
 	int optidx = 0;
 	pid_t pid;
-	poptContext pc;
+	poptContext pc = NULL;
 	const char *instruction_file;
 	const char **args;
 	const char *program;
@@ -415,18 +404,24 @@ int main(int argc, const char **argv)
 
 	if (argc == 1) {
 		poptPrintHelp(pc, stderr, 0);
-		return 1;
+		goto out;
 	}
 
 	while ((optidx = poptGetNextOpt(pc)) != -1) {
-		;;
+		switch (optidx) {
+		case POPT_ERROR_BADOPT:
+			fprintf(stderr, "\nInvalid option %s: %s\n\n",
+				poptBadOption(pc, 0), poptStrerror(optidx));
+			poptPrintUsage(pc, stderr, 0);
+			exit(1);
+		}
 	}
 
 	instruction_file = poptGetArg(pc);
 	args = poptGetArgs(pc);
 	if (args == NULL) {
 		poptPrintHelp(pc, stderr, 0);
-		return 1;
+		goto out;
 	}
 
 	program_args = (char * const *)discard_const_p(char *, args);
@@ -453,7 +448,7 @@ int main(int argc, const char **argv)
 			err(1, "Failed to fork");
 
 			/* Never reached */
-			return 1;
+			goto out;
 		case 0:
 
 			if(setsid()<0)
@@ -463,14 +458,14 @@ int main(int argc, const char **argv)
 			dup2(slave, STDOUT_FILENO);
 			dup2(slave, STDERR_FILENO);
 
-			sys_closefrom(STDERR_FILENO + 1);
+			closefrom(STDERR_FILENO + 1);
 
 			/* texpect <expect_instructions> <progname> [<args>] */
 			execvp(program, program_args);
 			err(1, "Failed to exec: %s", program);
 
 			/* Never reached */
-			return 1;
+			goto out;
 		default:
 			close(slave);
 			{
@@ -483,9 +478,13 @@ int main(int argc, const char **argv)
 				sigaction(SIGALRM, &sa, NULL);
 			}
 
+			poptFreeContext(pc);
 			return eval_parent(pid);
 	}
 
 	/* Never reached */
+
+out:
+	poptFreeContext(pc);
 	return 1;
 }
