@@ -27,6 +27,8 @@
 
 struct iovec;
 
+off_t sys_block_align_truncate(off_t len, off_t align);
+off_t sys_block_align(off_t len, off_t align);
 bool sys_valid_io_range(off_t offset, size_t length);
 bool sys_io_ranges_overlap(size_t c1, off_t o1,
 			   size_t c2, off_t o2);
@@ -39,5 +41,6 @@ ssize_t sys_pread(int fd, void *buf, size_t count, off_t off);
 ssize_t sys_pread_full(int fd, void *buf, size_t count, off_t off);
 ssize_t sys_pwrite(int fd, const void *buf, size_t count, off_t off);
 ssize_t sys_pwrite_full(int fd, const void *buf, size_t count, off_t off);
+ssize_t sys_write_full(int fd, const void *buf, size_t count);
 
 #endif
