@@ -21,8 +21,8 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "tevent_unix.h"
 #include "../replace/replace.h"
+#include "tevent_unix.h"
 
 bool tevent_req_is_unix_error(struct tevent_req *req, int *perrno)
 {
@@ -48,3 +48,26 @@ bool tevent_req_is_unix_error(struct tevent_req *req, int *perrno)
 	}
 	return true;
 }
+
+int tevent_req_simple_recv_unix(struct tevent_req *req)
+{
+	int err = 0;
+
+	/*
+	 * Ignore result of tevent_req_is_unix_error, we're only interested in
+	 * the status
+	 */
+	tevent_req_is_unix_error(req, &err);
+	tevent_req_received(req);
+	return err;
+}
+
+bool tevent_req_poll_unix(struct tevent_req *req, struct tevent_context *ev,
+			  int *err)
+{
+	bool ret = tevent_req_poll(req, ev);
+	if (!ret) {
+		*err = errno;
+	}
+	return ret;
+}
