@@ -20,7 +20,9 @@
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "includes.h"
+#include "replace.h"
+#include "lib/util/debug.h"
+#include "lib/util/fault.h"
 #include "librpc/gen_ndr/ndr_security.h"
 #include "libcli/security/security.h"
 
@@ -384,18 +386,6 @@ struct sec_desc_buf *make_sec_desc_buf(TALLOC_CTX *ctx, size_t len, struct secur
 	return dst;
 }
 
-/*******************************************************************
- Duplicates a struct sec_desc_buf structure.
-********************************************************************/
-
-struct sec_desc_buf *dup_sec_desc_buf(TALLOC_CTX *ctx, struct sec_desc_buf *src)
-{
-	if(src == NULL)
-		return NULL;
-
-	return make_sec_desc_buf( ctx, src->sd_size, src->sd);
-}
-
 /*
  * Determine if an struct security_ace is inheritable
  */
@@ -460,8 +450,6 @@ NTSTATUS se_create_child_secdesc(TALLOC_CTX *ctx,
 	unsigned int new_ace_list_ndx = 0, i;
 	bool set_inherited_flags = (parent_ctr->type & SEC_DESC_DACL_AUTO_INHERITED);
 
-	TALLOC_CTX *frame;
-
 	*ppsd = NULL;
 	*psize = 0;
 
@@ -484,14 +472,13 @@ NTSTATUS se_create_child_secdesc(TALLOC_CTX *ctx,
 		new_ace_list = NULL;
 	}
 
-	frame = talloc_stackframe();
-
 	for (i = 0; i < the_acl->num_aces; i++) {
 		const struct security_ace *ace = &the_acl->aces[i];
 		struct security_ace *new_ace = &new_ace_list[new_ace_list_ndx];
 		const struct dom_sid *ptrustee = &ace->trustee;
 		const struct dom_sid *creator = NULL;
 		uint8_t new_flags = ace->flags;
+		struct dom_sid_buf sidbuf1, sidbuf2;
 
 		if (!is_inheritable_ace(ace, container)) {
 			continue;
@@ -540,9 +527,9 @@ NTSTATUS se_create_child_secdesc(TALLOC_CTX *ctx,
 
 			DEBUG(5,("se_create_child_secdesc(): %s:%d/0x%02x/0x%08x"
 				 " inherited as %s:%d/0x%02x/0x%08x\n",
-				 dom_sid_string(frame, &ace->trustee),
+				 dom_sid_str_buf(&ace->trustee, &sidbuf1),
 				 ace->type, ace->flags, ace->access_mask,
-				 dom_sid_string(frame, &new_ace->trustee),
+				 dom_sid_str_buf(&new_ace->trustee, &sidbuf2),
 				 new_ace->type, new_ace->flags,
 				 new_ace->access_mask));
 
@@ -565,17 +552,15 @@ NTSTATUS se_create_child_secdesc(TALLOC_CTX *ctx,
 
 		DEBUG(5, ("se_create_child_secdesc(): %s:%d/0x%02x/0x%08x "
 			  " inherited as %s:%d/0x%02x/0x%08x\n",
-			  dom_sid_string(frame, &ace->trustee),
+			  dom_sid_str_buf(&ace->trustee, &sidbuf1),
 			  ace->type, ace->flags, ace->access_mask,
-			  dom_sid_string(frame, &new_ace->trustee),
+			  dom_sid_str_buf(&new_ace->trustee, &sidbuf2),
 			  new_ace->type, new_ace->flags,
 			  new_ace->access_mask));
 
 		new_ace_list_ndx++;
 	}
 
-	talloc_free(frame);
-
 	/*
 	 * remove duplicates
 	 */
