@@ -19,9 +19,11 @@
 
 #include "includes.h"
 #include "system/filesys.h"
-#include "libsmb/libsmb.h"
+#include "source3/include/client.h"
+#include "source3/libsmb/proto.h"
 #include "../libgpo/gpo.h"
 #include "libgpo/gpo_proto.h"
+#include "lib/util/string_wrappers.h"
 
 struct sync_context {
 	TALLOC_CTX *mem_ctx;
@@ -32,8 +34,7 @@ struct sync_context {
 	uint16_t attribute;
 };
 
-static NTSTATUS gpo_sync_func(const char *mnt,
-			  struct file_info *info,
+static NTSTATUS gpo_sync_func(struct file_info *info,
 			  const char *mask,
 			  void *state);
 
@@ -132,11 +133,10 @@ static NTSTATUS gpo_sync_files(struct sync_context *ctx)
 }
 
 /****************************************************************
- syncronisation call back
+ synchronisation callback
 ****************************************************************/
 
-static NTSTATUS gpo_sync_func(const char *mnt,
-			  struct file_info *info,
+static NTSTATUS gpo_sync_func(struct file_info *info,
 			  const char *mask,
 			  void *state)
 {
