@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    dcerpc utility functions
@@ -55,34 +55,34 @@ static const struct {
 	int num_protocols;
 	enum epm_protocol protseq[MAX_PROTSEQ];
 } transports[] = {
-	{ "ncacn_np",     NCACN_NP, 3, 
+	{ "ncacn_np",     NCACN_NP, 3,
 		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_SMB, EPM_PROTOCOL_NETBIOS }},
-	{ "ncacn_ip_tcp", NCACN_IP_TCP, 3, 
-		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_TCP, EPM_PROTOCOL_IP } }, 
-	{ "ncacn_http", NCACN_HTTP, 3, 
-		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_HTTP, EPM_PROTOCOL_IP } }, 
-	{ "ncadg_ip_udp", NCACN_IP_UDP, 3, 
+	{ "ncacn_ip_tcp", NCACN_IP_TCP, 3,
+		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_TCP, EPM_PROTOCOL_IP } },
+	{ "ncacn_http", NCACN_HTTP, 3,
+		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_HTTP, EPM_PROTOCOL_IP } },
+	{ "ncadg_ip_udp", NCACN_IP_UDP, 3,
 		{ EPM_PROTOCOL_NCADG, EPM_PROTOCOL_UDP, EPM_PROTOCOL_IP } },
-	{ "ncalrpc", NCALRPC, 2, 
+	{ "ncalrpc", NCALRPC, 2,
 		{ EPM_PROTOCOL_NCALRPC, EPM_PROTOCOL_NAMED_PIPE } },
-	{ "ncacn_unix_stream", NCACN_UNIX_STREAM, 2, 
+	{ "ncacn_unix_stream", NCACN_UNIX_STREAM, 2,
 		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_UNIX_DS } },
-	{ "ncadg_unix_dgram", NCADG_UNIX_DGRAM, 2, 
+	{ "ncadg_unix_dgram", NCADG_UNIX_DGRAM, 2,
 		{ EPM_PROTOCOL_NCADG, EPM_PROTOCOL_UNIX_DS } },
-	{ "ncacn_at_dsp", NCACN_AT_DSP, 3, 
+	{ "ncacn_at_dsp", NCACN_AT_DSP, 3,
 		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_APPLETALK, EPM_PROTOCOL_DSP } },
-	{ "ncadg_at_ddp", NCADG_AT_DDP, 3, 
+	{ "ncadg_at_ddp", NCADG_AT_DDP, 3,
 		{ EPM_PROTOCOL_NCADG, EPM_PROTOCOL_APPLETALK, EPM_PROTOCOL_DDP } },
-	{ "ncacn_vns_ssp", NCACN_VNS_SPP, 3, 
+	{ "ncacn_vns_ssp", NCACN_VNS_SPP, 3,
 		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_STREETTALK, EPM_PROTOCOL_VINES_SPP } },
-	{ "ncacn_vns_ipc", NCACN_VNS_IPC, 3, 
+	{ "ncacn_vns_ipc", NCACN_VNS_IPC, 3,
 		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_STREETTALK, EPM_PROTOCOL_VINES_IPC }, },
 	{ "ncadg_ipx", NCADG_IPX, 2,
 		{ EPM_PROTOCOL_NCADG, EPM_PROTOCOL_IPX },
 	},
 	{ "ncacn_spx", NCACN_SPX, 3,
-		/* I guess some MS programmer confused the identifier for 
-		 * EPM_PROTOCOL_UUID (0x0D or 13) with the one for 
+		/* I guess some MS programmer confused the identifier for
+		 * EPM_PROTOCOL_UUID (0x0D or 13) with the one for
 		 * EPM_PROTOCOL_SPX (0x13) here. -- jelmer*/
 		{ EPM_PROTOCOL_NCACN, EPM_PROTOCOL_NCALRPC, EPM_PROTOCOL_UUID },
 	},
@@ -134,18 +134,18 @@ const char *epm_floor_string(TALLOC_CTX *mem_ctx, struct epm_floor *epm_floor)
 
 	switch(epm_floor->lhs.protocol) {
 		case EPM_PROTOCOL_UUID:
-			status = dcerpc_floor_get_lhs_data(epm_floor, &syntax);
+			status = dcerpc_floor_get_uuid_full(epm_floor, &syntax);
 			if (NT_STATUS_IS_OK(status)) {
 				/* lhs is used: UUID */
 				struct GUID_txt_buf buf;
 
 				if (GUID_equal(&syntax.uuid, &ndr_transfer_syntax_ndr.uuid)) {
 					return "NDR";
-				} 
+				}
 
 				if (GUID_equal(&syntax.uuid, &ndr_transfer_syntax_ndr64.uuid)) {
 					return "NDR64";
-				} 
+				}
 
 				return talloc_asprintf(
 					mem_ctx,
@@ -153,7 +153,7 @@ const char *epm_floor_string(TALLOC_CTX *mem_ctx, struct epm_floor *epm_floor)
 					GUID_buf_string(&syntax.uuid, &buf),
 					syntax.if_version);
 			} else { /* IPX */
-				return talloc_asprintf(mem_ctx, "IPX:%s", 
+				return talloc_asprintf(mem_ctx, "IPX:%s",
 						data_blob_hex_string_upper(mem_ctx, &epm_floor->rhs.uuid.unknown));
 			}
 
@@ -194,13 +194,13 @@ const char *epm_floor_string(TALLOC_CTX *mem_ctx, struct epm_floor *epm_floor)
 			return "NB_IPX";
 
 		case EPM_PROTOCOL_HTTP:
-			return talloc_asprintf(mem_ctx, "HTTP:%d", epm_floor->rhs.http.port);
+			return talloc_asprintf(mem_ctx, "HTTP:%"PRIu16, epm_floor->rhs.http.port);
 
 		case EPM_PROTOCOL_TCP:
-			return talloc_asprintf(mem_ctx, "TCP:%d", epm_floor->rhs.tcp.port);
+			return talloc_asprintf(mem_ctx, "TCP:%"PRIu16, epm_floor->rhs.tcp.port);
 
 		case EPM_PROTOCOL_UDP:
-			return talloc_asprintf(mem_ctx, "UDP:%d", epm_floor->rhs.udp.port);
+			return talloc_asprintf(mem_ctx, "UDP:%"PRIu16, epm_floor->rhs.udp.port);
 
 		default:
 			return talloc_asprintf(mem_ctx, "UNK(%02x):", epm_floor->lhs.protocol);
@@ -894,14 +894,21 @@ _PUBLIC_ NTSTATUS dcerpc_binding_set_flags(struct dcerpc_binding *b,
 	return NT_STATUS_OK;
 }
 
-_PUBLIC_ NTSTATUS dcerpc_floor_get_lhs_data(const struct epm_floor *epm_floor,
-					    struct ndr_syntax_id *syntax)
+_PUBLIC_ NTSTATUS dcerpc_floor_get_uuid_full(const struct epm_floor *epm_floor,
+					     struct ndr_syntax_id *syntax)
 {
 	TALLOC_CTX *mem_ctx = talloc_init("floor_get_lhs_data");
 	struct ndr_pull *ndr;
 	enum ndr_err_code ndr_err;
 	uint16_t if_version=0;
 
+	*syntax = (struct ndr_syntax_id) { .if_version = 0, };
+
+	if (epm_floor->lhs.protocol != EPM_PROTOCOL_UUID) {
+		talloc_free(mem_ctx);
+		return NT_STATUS_INVALID_PARAMETER;
+	}
+
 	ndr = ndr_pull_init_blob(&epm_floor->lhs.lhs_data, mem_ctx);
 	if (ndr == NULL) {
 		talloc_free(mem_ctx);
@@ -923,6 +930,23 @@ _PUBLIC_ NTSTATUS dcerpc_floor_get_lhs_data(const struct epm_floor *epm_floor,
 
 	syntax->if_version = if_version;
 
+	TALLOC_FREE(ndr);
+
+	ndr = ndr_pull_init_blob(&epm_floor->rhs.uuid.unknown, mem_ctx);
+	if (ndr == NULL) {
+		talloc_free(mem_ctx);
+		return NT_STATUS_NO_MEMORY;
+	}
+	ndr->flags |= LIBNDR_FLAG_NOALIGN;
+
+	ndr_err = ndr_pull_uint16(ndr, NDR_SCALARS, &if_version);
+	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
+		talloc_free(mem_ctx);
+		return ndr_map_error2ntstatus(ndr_err);
+	}
+
+	syntax->if_version |= (((uint32_t)if_version) << 16) & 0xffff0000;
+
 	talloc_free(mem_ctx);
 
 	return NT_STATUS_OK;
@@ -982,20 +1006,43 @@ static bool dcerpc_floor_pack_rhs_if_version_data(
 	return true;
 }
 
+static NTSTATUS dcerpc_floor_pack_uuid_full(TALLOC_CTX *mem_ctx,
+					    struct epm_floor *floor,
+					    const struct ndr_syntax_id *syntax)
+{
+	bool ok;
+
+	floor->lhs.protocol = EPM_PROTOCOL_UUID;
+
+	floor->lhs.lhs_data = dcerpc_floor_pack_lhs_data(mem_ctx, syntax);
+	if (floor->lhs.lhs_data.data == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	ok = dcerpc_floor_pack_rhs_if_version_data(mem_ctx, syntax,
+				&floor->rhs.uuid.unknown);
+	if (!ok) {
+		data_blob_free(&floor->lhs.lhs_data);
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	return NT_STATUS_OK;
+}
+
 char *dcerpc_floor_get_rhs_data(TALLOC_CTX *mem_ctx, struct epm_floor *epm_floor)
 {
 	switch (epm_floor->lhs.protocol) {
 	case EPM_PROTOCOL_TCP:
 		if (epm_floor->rhs.tcp.port == 0) return NULL;
-		return talloc_asprintf(mem_ctx, "%d", epm_floor->rhs.tcp.port);
+		return talloc_asprintf(mem_ctx, "%"PRIu16, epm_floor->rhs.tcp.port);
 
 	case EPM_PROTOCOL_UDP:
 		if (epm_floor->rhs.udp.port == 0) return NULL;
-		return talloc_asprintf(mem_ctx, "%d", epm_floor->rhs.udp.port);
+		return talloc_asprintf(mem_ctx, "%"PRIu16, epm_floor->rhs.udp.port);
 
 	case EPM_PROTOCOL_HTTP:
 		if (epm_floor->rhs.http.port == 0) return NULL;
-		return talloc_asprintf(mem_ctx, "%d", epm_floor->rhs.http.port);
+		return talloc_asprintf(mem_ctx, "%"PRIu16, epm_floor->rhs.http.port);
 
 	case EPM_PROTOCOL_IP:
 		return talloc_strdup(mem_ctx, epm_floor->rhs.ip.ipaddr);
@@ -1022,10 +1069,10 @@ char *dcerpc_floor_get_rhs_data(TALLOC_CTX *mem_ctx, struct epm_floor *epm_floor
 		return NULL;
 
 	case EPM_PROTOCOL_VINES_SPP:
-		return talloc_asprintf(mem_ctx, "%d", epm_floor->rhs.vines_spp.port);
+		return talloc_asprintf(mem_ctx, "%"PRIu16, epm_floor->rhs.vines_spp.port);
 
 	case EPM_PROTOCOL_VINES_IPC:
-		return talloc_asprintf(mem_ctx, "%d", epm_floor->rhs.vines_ipc.port);
+		return talloc_asprintf(mem_ctx, "%"PRIu16, epm_floor->rhs.vines_ipc.port);
 
 	case EPM_PROTOCOL_STREETTALK:
 		return talloc_strdup(mem_ctx, epm_floor->rhs.streettalk.streettalk);
@@ -1045,8 +1092,8 @@ char *dcerpc_floor_get_rhs_data(TALLOC_CTX *mem_ctx, struct epm_floor *epm_floor
 	return NULL;
 }
 
-static NTSTATUS dcerpc_floor_set_rhs_data(TALLOC_CTX *mem_ctx, 
-					  struct epm_floor *epm_floor,  
+static NTSTATUS dcerpc_floor_set_rhs_data(TALLOC_CTX *mem_ctx,
+					  struct epm_floor *epm_floor,
 					  const char *data)
 {
 	if (data == NULL) {
@@ -1135,7 +1182,7 @@ enum dcerpc_transport_t dcerpc_transport_by_endpoint_protocol(int prot)
 
 	/* Find a transport that has 'prot' as 4th protocol */
 	for (i=0;i<ARRAY_SIZE(transports);i++) {
-		if (transports[i].num_protocols >= 2 && 
+		if (transports[i].num_protocols >= 2 &&
 			transports[i].protseq[1] == prot) {
 			return transports[i].transport;
 		}
@@ -1153,7 +1200,7 @@ _PUBLIC_ enum dcerpc_transport_t dcerpc_transport_by_tower(const struct epm_towe
 	for (i=0;i<ARRAY_SIZE(transports);i++) {
 		int j;
 		if (transports[i].num_protocols != tower->num_floors - 2) {
-			continue; 
+			continue;
 		}
 
 		for (j = 0; j < transports[i].num_protocols && j < MAX_PROTSEQ; j++) {
@@ -1238,7 +1285,7 @@ _PUBLIC_ NTSTATUS dcerpc_binding_from_tower(TALLOC_CTX *mem_ctx,
 	}
 
 	/* Set abstract syntax */
-	status = dcerpc_floor_get_lhs_data(&tower->floors[0], &abstract_syntax);
+	status = dcerpc_floor_get_uuid_full(&tower->floors[0], &abstract_syntax);
 	if (!NT_STATUS_IS_OK(status)) {
 		talloc_free(b);
 		return status;
@@ -1402,25 +1449,21 @@ _PUBLIC_ NTSTATUS dcerpc_binding_build_tower(TALLOC_CTX *mem_ctx,
 	}
 
 	/* Floor 0 */
-	tower->floors[0].lhs.protocol = EPM_PROTOCOL_UUID;
-
 	abstract_syntax = dcerpc_binding_get_abstract_syntax(binding);
-	tower->floors[0].lhs.lhs_data = dcerpc_floor_pack_lhs_data(tower->floors,
-								   &abstract_syntax);
-
-	if (!dcerpc_floor_pack_rhs_if_version_data(
-		    tower->floors, &abstract_syntax,
-		    &tower->floors[0].rhs.uuid.unknown)) {
-		return NT_STATUS_NO_MEMORY;
+	status = dcerpc_floor_pack_uuid_full(tower->floors,
+					     &tower->floors[0],
+					     &abstract_syntax);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
 	}
 
 	/* Floor 1 */
-	tower->floors[1].lhs.protocol = EPM_PROTOCOL_UUID;
-
-	tower->floors[1].lhs.lhs_data = dcerpc_floor_pack_lhs_data(tower->floors, 
-								&ndr_transfer_syntax_ndr);
-
-	tower->floors[1].rhs.uuid.unknown = data_blob_talloc_zero(tower->floors, 2);
+	status = dcerpc_floor_pack_uuid_full(tower->floors,
+					     &tower->floors[1],
+					     &ndr_transfer_syntax_ndr);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
 
 	/* Floor 2 to num_protocols */
 	for (i = 0; i < num_protocols; i++) {
