@@ -14,7 +14,7 @@ push @ISA, qw(Exporter);
 
 use strict;
 use warnings;
-use Parse::Pidl::Typelist qw(hasType getType mapTypeName typeHasBody);
+use Parse::Pidl::Typelist qw(hasType getType mapTypeName mapTypeSpecifier typeHasBody);
 use Parse::Pidl::Util qw(has_property
 			 ParseExpr
 			 ParseExprExt
@@ -57,7 +57,7 @@ sub append_prefix($$)
 			$pointers++;
 		} elsif ($l->{TYPE} eq "ARRAY") {
 			$arrays++;
-			if (($pointers == 0) and 
+			if (($pointers == 0) and
 			    (not $l->{IS_FIXED}) and
 			    (not $l->{IS_INLINE})) {
 				return get_value_of($var_name);
@@ -68,7 +68,7 @@ sub append_prefix($$)
 			}
 		}
 	}
-	
+
 	return $var_name;
 }
 
@@ -102,8 +102,8 @@ sub is_public_struct
 }
 
 ####################################
-# defer() is like pidl(), but adds to 
-# a deferred buffer which is then added to the 
+# defer() is like pidl(), but adds to
+# a deferred buffer which is then added to the
 # output buffer at the end of the structure/union/function
 # This is needed to cope with code that must be pushed back
 # to the end of a block of elements
@@ -159,7 +159,7 @@ sub start_flags($$$)
 	if (defined $flags) {
 		$self->pidl("{");
 		$self->indent;
-		$self->pidl("uint32_t _flags_save_$e->{TYPE} = $ndr->flags;");
+		$self->pidl("libndr_flags _flags_save_$e->{TYPE} = $ndr->flags;");
 		$self->pidl("ndr_set_flags(&$ndr->flags, $flags);");
 	}
 }
@@ -235,7 +235,7 @@ sub check_fully_dereferenced($$)
 				last;
 			}
 		}
-		
+
 		return($origvar) unless (defined($var));
 		my $e;
 		foreach (@{$element->{PARENT}->{ELEMENTS}}) {
@@ -258,7 +258,7 @@ sub check_fully_dereferenced($$)
 		warning($element->{ORIGINAL}, "Got pointer for `$e->{NAME}', expected fully dereferenced variable") if ($nump > length($ptr));
 		return ($origvar);
 	}
-}	
+}
 
 sub check_null_pointer($$$$)
 {
@@ -279,7 +279,7 @@ sub check_null_pointer($$$$)
 				last;
 			}
 		}
-		
+
 		if (defined($var)) {
 			my $e;
 			# lookup ptr in $e
@@ -295,7 +295,7 @@ sub check_null_pointer($$$$)
 			# See if pointer at pointer level $level
 			# needs to be checked.
 			foreach my $l (@{$e->{LEVELS}}) {
-				if ($l->{TYPE} eq "POINTER" and 
+				if ($l->{TYPE} eq "POINTER" and
 					$l->{POINTER_INDEX} == length($ptr)) {
 					# No need to check ref pointers
 					$check = ($l->{POINTER_TYPE} ne "ref");
@@ -310,7 +310,7 @@ sub check_null_pointer($$$$)
 			warning($element, "unknown dereferenced expression `$expandedvar'");
 			$check = 1;
 		}
-		
+
 		$print_fn->("if ($ptr$expandedvar == NULL) $return") if $check;
 	}
 }
@@ -371,7 +371,8 @@ sub ParseArrayPullGetSize($$$$$$)
 		} else {
 			$self->pidl("if ($array_size < $low || $array_size > $high) {");
 		}
-		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_RANGE, \"value out of range\");");
+		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_RANGE, \"value (%\"PRIu32\") out of range (%\"PRIu32\" - %\"PRIu32\")\", $array_size, (uint32_t)($low), (uint32_t)($high));");
+
 		$self->pidl("}");
 	}
 
@@ -410,7 +411,7 @@ sub ParseArrayPullGetLength($$$$$$;$)
 		} else {
 			$self->pidl("if ($array_length < $low || $array_length > $high) {");
 		}
-		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_RANGE, \"value out of range\");");
+		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_RANGE, \"value (%\"PRIu32\") out of range (%\"PRIu32\" - %\"PRIu32\")\", $array_length, (uint32_t)($low), (uint32_t)($high));");
 		$self->pidl("}");
 	}
 
@@ -437,7 +438,7 @@ sub ParseArrayPullHeader($$$$$$)
 	if ($array_length ne $array_size) {
 		$self->pidl("if ($array_length > $array_size) {");
 		$self->indent;
-		$self->pidl("return ndr_pull_error($ndr, NDR_ERR_ARRAY_SIZE, \"Bad array size %u should exceed array length %u\", $array_size, $array_length);");
+		$self->pidl("return ndr_pull_error($ndr, NDR_ERR_ARRAY_SIZE, \"Bad array size %\"PRIu32\": should exceed array length %\"PRIu32\"\", $array_size, $array_length);");
 		$self->deindent;
 		$self->pidl("}");
 	}
@@ -464,7 +465,7 @@ sub ParseArrayPullHeader($$$$$$)
 	if ($l->{IS_VARYING} and (defined($l->{LENGTH_IS}) or not $l->{IS_ZERO_TERMINATED})) {
 		$self->defer("if ($var_name) {");
 		$self->defer_indent;
-		my $length = ParseExprExt($l->{LENGTH_IS}, $env, $e->{ORIGINAL}, 
+		my $length = ParseExprExt($l->{LENGTH_IS}, $env, $e->{ORIGINAL},
 			check_null_pointer($e, $env, sub { $self->defer(shift); },
 					   "return ndr_pull_error($ndr, NDR_ERR_INVALID_POINTER, \"NULL Pointer for length_is()\");"),
 			check_fully_dereferenced($e, $env));
@@ -602,7 +603,8 @@ sub ParseSubcontextPullStart($$$$$)
 	$self->pidl("{");
 	$self->indent;
 	$self->pidl("struct ndr_pull *$subndr;");
-	$self->pidl("NDR_CHECK(ndr_pull_subcontext_start($ndr, &$subndr, $l->{HEADER_SIZE}, $subcontext_size));");
+	$self->pidl("ssize_t sub_size = $subcontext_size;");
+	$self->pidl("NDR_CHECK(ndr_pull_subcontext_start($ndr, &$subndr, $l->{HEADER_SIZE}, sub_size));");
 
 	if (defined $l->{COMPRESSION}) {
 		$subndr = $self->ParseCompressionPullStart($e, $l, $subndr, $env);
@@ -621,7 +623,7 @@ sub ParseSubcontextPullEnd($$$$$)
 		$self->ParseCompressionPullEnd($e, $l, $subndr, $env);
 	}
 
-	$self->pidl("NDR_CHECK(ndr_pull_subcontext_end($ndr, $subndr, $l->{HEADER_SIZE}, $subcontext_size));");
+	$self->pidl("NDR_CHECK(ndr_pull_subcontext_end($ndr, $subndr, $l->{HEADER_SIZE}, sub_size));");
 	$self->deindent;
 	$self->pidl("}");
 }
@@ -644,7 +646,7 @@ sub ParseElementPushLevel
 		} elsif ($l->{TYPE} eq "POINTER") {
 			$self->ParsePtrPush($e, $l, $ndr, $var_name);
 		} elsif ($l->{TYPE} eq "ARRAY") {
-			my $length = $self->ParseArrayPushHeader($e, $l, $ndr, $var_name, $env); 
+			my $length = $self->ParseArrayPushHeader($e, $l, $ndr, $var_name, $env);
 
 			my $nl = GetNextLevel($e, $l);
 
@@ -659,7 +661,7 @@ sub ParseElementPushLevel
 			} elsif (has_fast_array($e,$l)) {
 				$self->pidl("NDR_CHECK(ndr_push_array_$nl->{DATA_TYPE}($ndr, $ndr_flags, $var_name, $length));");
 				return;
-			} 
+			}
 		} elsif ($l->{TYPE} eq "DATA") {
 			$self->ParseDataPush($e, $l, $ndr, $var_name, $primitives, $deferred);
 		} elsif ($l->{TYPE} eq "TYPEDEF") {
@@ -905,13 +907,13 @@ sub ParseElementPrint($$$$$)
 			my $length;
 
 			if ($l->{IS_CONFORMANT} or $l->{IS_VARYING}) {
-				$var_name = get_pointer_to($var_name); 
+				$var_name = get_pointer_to($var_name);
 			}
-			
+
 			if ($l->{IS_ZERO_TERMINATED} and not defined($l->{LENGTH_IS})) {
 				$length = "ndr_string_length($var_name, sizeof(*$var_name))";
 			} else {
-				$length = ParseExprExt($l->{LENGTH_IS}, $env, $e->{ORIGINAL}, 
+				$length = ParseExprExt($l->{LENGTH_IS}, $env, $e->{ORIGINAL},
 							check_null_pointer($e, $env, sub { $self->pidl(shift); }, "return;"), check_fully_dereferenced($e, $env));
 			}
 
@@ -925,7 +927,7 @@ sub ParseElementPrint($$$$$)
 			} else {
 				my $counter = "cntr_$e->{NAME}_$l->{LEVEL_INDEX}";
 
-				$self->pidl("$ndr->print($ndr, \"\%s: ARRAY(\%d)\", \"$e->{NAME}\", (int)$length);");
+				$self->pidl("$ndr->print($ndr, \"%s: ARRAY(%\"PRIu32\")\", \"$e->{NAME}\", (uint32_t)($length));");
 				$self->pidl("$ndr->depth++;");
 				$self->pidl("for ($counter = 0; $counter < ($length); $counter++) {");
 				$self->indent;
@@ -935,10 +937,10 @@ sub ParseElementPrint($$$$$)
 		} elsif ($l->{TYPE} eq "DATA") {
 			$self->ParseDataPrint($e, $l, $ndr, $var_name);
 		} elsif ($l->{TYPE} eq "SWITCH") {
-			my $switch_var = ParseExprExt($l->{SWITCH_IS}, $env, $e->{ORIGINAL}, 
+			my $switch_var = ParseExprExt($l->{SWITCH_IS}, $env, $e->{ORIGINAL},
 						check_null_pointer($e, $env, sub { $self->pidl(shift); }, "return;"), check_fully_dereferenced($e, $env));
 			$self->pidl("ndr_print_set_switch_value($ndr, " . get_pointer_to($var_name) . ", $switch_var);");
-		} 
+		}
 	}
 
 	foreach my $l (reverse @{$e->{LEVELS}}) {
@@ -975,7 +977,7 @@ sub ParseElementPrint($$$$$)
 sub ParseSwitchPull($$$$$$)
 {
 	my($self,$e,$l,$ndr,$var_name,$env) = @_;
-	my $switch_var = ParseExprExt($l->{SWITCH_IS}, $env, $e->{ORIGINAL}, 
+	my $switch_var = ParseExprExt($l->{SWITCH_IS}, $env, $e->{ORIGINAL},
 		check_null_pointer($e, $env, sub { $self->pidl(shift); },
 				   "return ndr_pull_error($ndr, NDR_ERR_INVALID_POINTER, \"NULL Pointer for switch_is()\");"),
 		check_fully_dereferenced($e, $env));
@@ -989,7 +991,7 @@ sub ParseSwitchPull($$$$$$)
 sub ParseSwitchPush($$$$$$)
 {
 	my($self,$e,$l,$ndr,$var_name,$env) = @_;
-	my $switch_var = ParseExprExt($l->{SWITCH_IS}, $env, $e->{ORIGINAL}, 
+	my $switch_var = ParseExprExt($l->{SWITCH_IS}, $env, $e->{ORIGINAL},
 		check_null_pointer($e, $env, sub { $self->pidl(shift); },
 				   "return ndr_push_error($ndr, NDR_ERR_INVALID_POINTER, \"NULL Pointer for switch_is()\");"),
 		check_fully_dereferenced($e, $env));
@@ -1036,7 +1038,20 @@ sub ParseDataPull($$$$$$$)
 			} else {
 				$self->pidl("if ($var_name < $low || $var_name > $high) {");
 			}
-			$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_RANGE, \"value out of range\");");
+
+			my $data_type = mapTypeName($l->{DATA_TYPE});
+			my $fmt = mapTypeSpecifier($data_type);
+
+			if (!defined($fmt)) {
+				if (getType($l->{DATA_TYPE})->{DATA}->{TYPE} eq "ENUM") {
+					$data_type = "int";
+					$fmt = "d";
+				} else {
+					die("Format ($data_type) not supported");
+				}
+			}
+
+			$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_RANGE, \"value (%$fmt) out of range (%$fmt - %$fmt)\", ($data_type)($var_name), ($data_type)($low), ($data_type)($high));");
 			$self->pidl("}");
 		}
 	} else {
@@ -1070,14 +1085,14 @@ sub CalcNdrFlags($$$)
 	my $scalars = 0;
 	my $buffers = 0;
 
-	# Add NDR_SCALARS if this one is deferred 
+	# Add NDR_SCALARS if this one is deferred
 	# and deferreds may be pushed
 	$scalars = 1 if ($l->{IS_DEFERRED} and $deferred);
 
-	# Add NDR_SCALARS if this one is not deferred and 
+	# Add NDR_SCALARS if this one is not deferred and
 	# primitives may be pushed
 	$scalars = 1 if (!$l->{IS_DEFERRED} and $primitives);
-	
+
 	# Add NDR_BUFFERS if this one contains deferred stuff
 	# and deferreds may be pushed
 	$buffers = 1 if ($l->{CONTAINS_DEFERRED} and $deferred);
@@ -1105,7 +1120,7 @@ sub ParseMemCtxPullFlags($$$$)
 		my $nl = GetNextLevel($e, $l);
 		return undef if ($nl->{TYPE} eq "PIPE");
 		return undef if ($nl->{TYPE} eq "ARRAY");
-		return undef if (($nl->{TYPE} eq "DATA") and ($nl->{DATA_TYPE} eq "string"));
+		return undef if (($nl->{TYPE} eq "DATA") and (Parse::Pidl::Typelist::is_string_type($nl->{DATA_TYPE})));
 
 		if ($l->{LEVEL} eq "TOP") {
 			$mem_flags = "LIBNDR_FLAG_REF_ALLOC";
@@ -1241,7 +1256,7 @@ sub ParseElementPullLevel
 			$self->deindent;
 			$self->pidl("}");
 		}
-	} elsif ($l->{TYPE} eq "ARRAY" and 
+	} elsif ($l->{TYPE} eq "ARRAY" and
 			not has_fast_array($e,$l) and not is_charset_array($e, $l)) {
 		my $length = $array_length;
 		my $counter = "cntr_$e->{NAME}_$l->{LEVEL_INDEX}";
@@ -1343,21 +1358,21 @@ sub ParsePtrPull($$$$$)
 
 	my $nl = GetNextLevel($e, $l);
 	my $next_is_array = ($nl->{TYPE} eq "ARRAY");
-	my $next_is_string = (($nl->{TYPE} eq "DATA") and 
-						 ($nl->{DATA_TYPE} eq "string"));
+	my $next_is_string = (($nl->{TYPE} eq "DATA") and
+						 (Parse::Pidl::Typelist::is_string_type($nl->{DATA_TYPE})));
 
 	if ($l->{POINTER_TYPE} eq "ref" and $l->{LEVEL} eq "TOP") {
 
 		if (!$next_is_array and !$next_is_string) {
 			$self->pidl("if ($ndr->flags & LIBNDR_FLAG_REF_ALLOC) {");
-			$self->pidl("\tNDR_PULL_ALLOC($ndr, $var_name);"); 
+			$self->pidl("\tNDR_PULL_ALLOC($ndr, $var_name);");
 			$self->pidl("}");
 		}
 
 		return;
 	} elsif ($l->{POINTER_TYPE} eq "ref" and $l->{LEVEL} eq "EMBEDDED") {
 		$self->pidl("NDR_CHECK(ndr_pull_ref_ptr($ndr, &_ptr_$e->{NAME}));");
-	} elsif (($l->{POINTER_TYPE} eq "unique") or 
+	} elsif (($l->{POINTER_TYPE} eq "unique") or
 		 ($l->{POINTER_TYPE} eq "relative") or
 		 ($l->{POINTER_TYPE} eq "full")) {
 		$self->pidl("NDR_CHECK(ndr_pull_generic_ptr($ndr, &_ptr_$e->{NAME}));");
@@ -1379,10 +1394,10 @@ sub ParsePtrPull($$$$$)
 	        # allocation, as we forced it to NULL just above, and
 	        # we may not know the declared type anyway.
 	} else {
-	        # Don't do this for arrays, they're allocated at the actual level 
+	        # Don't do this for arrays, they're allocated at the actual level
 	        # of the array
-	        unless ($next_is_array or $next_is_string) { 
-		       $self->pidl("NDR_PULL_ALLOC($ndr, $var_name);"); 
+	        unless ($next_is_array or $next_is_string) {
+		       $self->pidl("NDR_PULL_ALLOC($ndr, $var_name);");
 		} else {
 		       # FIXME: Yes, this is nasty.
 		       # We allocate an array twice
@@ -1434,10 +1449,10 @@ sub ParseStructPushPrimitives($$$$$)
 	if (defined($struct->{SURROUNDING_ELEMENT})) {
 		my $e = $struct->{SURROUNDING_ELEMENT};
 
-		if (defined($e->{LEVELS}[0]) and 
+		if (defined($e->{LEVELS}[0]) and
 			$e->{LEVELS}[0]->{TYPE} eq "ARRAY") {
 			my $size;
-			
+
 			if ($e->{LEVELS}[0]->{IS_ZERO_TERMINATED}) {
 				if (has_property($e, "charset")) {
 					$size = "ndr_charset_length($varname->$e->{NAME}, CH_$e->{PROPERTIES}->{charset})";
@@ -1486,7 +1501,7 @@ sub ParseStructPushDeferred($$$$)
 sub ParseStructPush($$$$)
 {
 	my ($self, $struct, $ndr, $varname) = @_;
-	
+
 	return unless defined($struct->{ELEMENTS});
 
 	my $env = GenerateStructEnv($struct, $varname);
@@ -1566,7 +1581,7 @@ sub ParseEnumPrint($$$$$)
 
 	$self->deindent;
 	$self->pidl("}");
-	
+
 	$self->pidl("ndr_print_enum($ndr, name, \"$enum->{TYPE}\", val, $varname);");
 
 	$self->end_flags($enum, $ndr);
@@ -1575,7 +1590,7 @@ sub ParseEnumPrint($$$$$)
 sub DeclEnum($$$$)
 {
 	my ($e,$t,$name,$varname) = @_;
-	return "enum $name " . 
+	return "enum $name " .
 		($t eq "pull"?"*":"") . $varname;
 }
 
@@ -1658,7 +1673,7 @@ sub ParseBitmapPrint($$$$$)
 sub DeclBitmap($$$$)
 {
 	my ($e,$t,$name,$varname) = @_;
-	return mapTypeName(Parse::Pidl::Typelist::bitmap_type_fn($e)) . 
+	return mapTypeName(Parse::Pidl::Typelist::bitmap_type_fn($e)) .
 		($t eq "pull"?" *":" ") . $varname;
 }
 
@@ -1687,7 +1702,7 @@ sub ParseStructPrint($$$$$)
 	$self->start_flags($struct, $ndr);
 
 	$self->pidl("$ndr->depth++;");
-	
+
 	$self->ParseElementPrint($_, $ndr, $env->{$_->{NAME}}, $env)
 		foreach (@{$struct->{ELEMENTS}});
 	$self->pidl("$ndr->depth--;");
@@ -1705,7 +1720,7 @@ sub DeclarePtrVariables($$)
 
 	foreach my $l (@{$e->{LEVELS}}) {
 		my $size = 32;
-		if ($l->{TYPE} eq "POINTER" and 
+		if ($l->{TYPE} eq "POINTER" and
 			not ($l->{POINTER_TYPE} eq "ref" and $l->{LEVEL} eq "TOP")) {
 			if ($l->{POINTER_TYPE} eq "relative_short") {
 				$size = 16;
@@ -1878,7 +1893,7 @@ sub DeclStruct($$$$)
 sub ArgsStructNdrSize($$$)
 {
 	my ($d, $name, $varname) = @_;
-	return "const struct $name *$varname, int flags";
+	return "const struct $name *$varname, libndr_flags flags";
 }
 
 $typefamily{STRUCT} = {
@@ -1954,7 +1969,7 @@ sub ParseUnionPushPrimitives($$$$)
 	}
 	if (! $have_default) {
 		$self->pidl("default:");
-		$self->pidl("\treturn ndr_push_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value \%u\", level);");
+		$self->pidl("\treturn ndr_push_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value %\"PRIu32, level);");
 	}
 	$self->deindent;
 	$self->pidl("}");
@@ -1989,7 +2004,7 @@ sub ParseUnionPushDeferred($$$$)
 	}
 	if (! $have_default) {
 		$self->pidl("default:");
-		$self->pidl("\treturn ndr_push_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value \%u\", level);");
+		$self->pidl("\treturn ndr_push_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value %\"PRIu32, level);");
 	}
 	$self->deindent;
 	$self->pidl("}");
@@ -2085,9 +2100,17 @@ sub ParseUnionPullPrimitives($$$$$)
 			$self->pidl("NDR_CHECK(ndr_pull_union_align($ndr, $e->{ALIGN}));");
 		}
 
+		my $data_type = mapTypeName($switch_type);
+		my $fmt = mapTypeSpecifier($data_type);
+
+		if (!defined($fmt)) {
+			$data_type = "int";
+			$fmt = "%d";
+		}
+
 		$self->pidl("NDR_CHECK(ndr_pull_$switch_type($ndr, NDR_SCALARS, &_level));");
-		$self->pidl("if (_level != level) {"); 
-		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value %u for $varname at \%s\", _level, __location__);");
+		$self->pidl("if (_level != level) {");
+		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value %$fmt for $varname at \%s\", ($data_type)_level, __location__);");
 		$self->pidl("}");
 	}
 
@@ -2105,7 +2128,7 @@ sub ParseUnionPullPrimitives($$$$$)
 	foreach my $el (@{$e->{ELEMENTS}}) {
 		if ($el->{CASE} eq "default") {
 			$have_default = 1;
-		} 
+		}
 		$self->pidl("$el->{CASE}: {");
 
 		if ($el->{TYPE} ne "EMPTY") {
@@ -2124,7 +2147,7 @@ sub ParseUnionPullPrimitives($$$$$)
 	}
 	if (! $have_default) {
 		$self->pidl("default:");
-		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value \%u at \%s\", level, __location__);");
+		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value %\"PRIu32\" at %s\", level, __location__);");
 	}
 	$self->deindent;
 	$self->pidl("}");
@@ -2140,7 +2163,7 @@ sub ParseUnionPullDeferred($$$$)
 	foreach my $el (@{$e->{ELEMENTS}}) {
 		if ($el->{CASE} eq "default") {
 			$have_default = 1;
-		} 
+		}
 
 		$self->pidl("$el->{CASE}:");
 		if ($el->{TYPE} ne "EMPTY") {
@@ -2158,7 +2181,7 @@ sub ParseUnionPullDeferred($$$$)
 	}
 	if (! $have_default) {
 		$self->pidl("default:");
-		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value \%u at \%s\", level, __location__);");
+		$self->pidl("\treturn ndr_pull_error($ndr, NDR_ERR_BAD_SWITCH, \"Bad switch value %\"PRIu32\" at %s\", level, __location__);");
 	}
 	$self->deindent;
 	$self->pidl("}");
@@ -2232,7 +2255,7 @@ sub DeclUnion($$$$)
 sub ArgsUnionNdrSize($$)
 {
 	my ($d,$name) = @_;
-	return "const union $name *r, uint32_t level, int flags";
+	return "const union $name *r, uint32_t level, libndr_flags flags";
 }
 
 $typefamily{UNION} = {
@@ -2243,7 +2266,7 @@ $typefamily{UNION} = {
 	SIZE_FN_ARGS => \&ArgsUnionNdrSize,
 	SIZE_FN_BODY => \&ParseUnionNdrSize,
 };
-	
+
 #####################################################################
 # parse a typedef - push side
 sub ParseTypedefPush($$$$)
@@ -2291,7 +2314,7 @@ sub ParseTypedefNdrSize($$$$)
 sub DeclTypedef($$$$)
 {
 	my ($e, $t, $name, $varname) = @_;
-	
+
 	return $typefamily{$e->{DATA}->{TYPE}}->{DECL}->($e->{DATA}, $t, $name, $varname);
 }
 
@@ -2324,7 +2347,7 @@ sub ParsePipePushChunk($$)
 
 	my $args = $typefamily{$struct->{TYPE}}->{DECL}->($struct, "push", $name, $varname);
 
-	$self->fn_declare("push", $struct, "enum ndr_err_code ndr_push_$name(struct ndr_push *$ndr, int ndr_flags, $args)") or return;
+	$self->fn_declare("push", $struct, "enum ndr_err_code ndr_push_$name(struct ndr_push *$ndr, ndr_flags_type ndr_flags, $args)") or return;
 
 	return if has_property($t, "nopush");
 
@@ -2357,7 +2380,7 @@ sub ParsePipePullChunk($$)
 
 	my $args = $typefamily{$struct->{TYPE}}->{DECL}->($struct, "pull", $name, $varname);
 
-	$self->fn_declare("pull", $struct, "enum ndr_err_code ndr_pull_$name(struct ndr_pull *$ndr, int ndr_flags, $args)") or return;
+	$self->fn_declare("pull", $struct, "enum ndr_err_code ndr_pull_$name(struct ndr_pull *$ndr, ndr_flags_type ndr_flags, $args)") or return;
 
 	return if has_property($struct, "nopull");
 
@@ -2410,11 +2433,11 @@ sub ParseFunctionPrint($$)
 	my($self, $fn) = @_;
 	my $ndr = "ndr";
 
-	$self->pidl_hdr("void ndr_print_$fn->{NAME}(struct ndr_print *$ndr, const char *name, int flags, const struct $fn->{NAME} *r);");
+	$self->pidl_hdr("void ndr_print_$fn->{NAME}(struct ndr_print *$ndr, const char *name, ndr_flags_type flags, const struct $fn->{NAME} *r);");
 
 	return if has_property($fn, "noprint");
 
-	$self->pidl("_PUBLIC_ void ndr_print_$fn->{NAME}(struct ndr_print *$ndr, const char *name, int flags, const struct $fn->{NAME} *r)");
+	$self->pidl("_PUBLIC_ void ndr_print_$fn->{NAME}(struct ndr_print *$ndr, const char *name, ndr_flags_type flags, const struct $fn->{NAME} *r)");
 	$self->pidl("{");
 	$self->indent;
 
@@ -2445,7 +2468,7 @@ sub ParseFunctionPrint($$)
 	$self->pidl("$ndr->depth--;");
 	$self->deindent;
 	$self->pidl("}");
-	
+
 	$self->pidl("if (flags & NDR_OUT) {");
 	$self->indent;
 	$self->pidl("ndr_print_struct($ndr, \"out\", \"$fn->{NAME}\");");
@@ -2463,7 +2486,7 @@ sub ParseFunctionPrint($$)
 	$self->pidl("$ndr->depth--;");
 	$self->deindent;
 	$self->pidl("}");
-	
+
 	$self->pidl("$ndr->depth--;");
 	$self->deindent;
 	$self->pidl("}");
@@ -2473,18 +2496,18 @@ sub ParseFunctionPrint($$)
 #####################################################################
 # parse a function
 sub ParseFunctionPush($$)
-{ 
+{
 	my($self, $fn) = @_;
 	my $ndr = "ndr";
 
-	$self->fn_declare("push", $fn, "enum ndr_err_code ndr_push_$fn->{NAME}(struct ndr_push *$ndr, int flags, const struct $fn->{NAME} *r)") or return;
+	$self->fn_declare("push", $fn, "enum ndr_err_code ndr_push_$fn->{NAME}(struct ndr_push *$ndr, ndr_flags_type flags, const struct $fn->{NAME} *r)") or return;
 
 	return if has_property($fn, "nopush");
 
 	$self->pidl("{");
 	$self->indent;
 
-	foreach my $e (@{$fn->{ELEMENTS}}) { 
+	foreach my $e (@{$fn->{ELEMENTS}}) {
 		$self->DeclareArrayVariables($e);
 	}
 
@@ -2533,7 +2556,7 @@ sub ParseFunctionPush($$)
 	if ($fn->{RETURN_TYPE}) {
 		$self->pidl("NDR_CHECK(ndr_push_$fn->{RETURN_TYPE}($ndr, NDR_SCALARS, r->out.result));");
 	}
-    
+
 	$self->deindent;
 	$self->pidl("}");
 	$self->pidl("return NDR_ERR_SUCCESS;");
@@ -2547,8 +2570,8 @@ sub AllocateArrayLevel($$$$$$)
 	my ($self,$e,$l,$ndr,$var,$size) = @_;
 
 	my $pl = GetPrevLevel($e, $l);
-	if (defined($pl) and 
-	    $pl->{TYPE} eq "POINTER" and 
+	if (defined($pl) and
+	    $pl->{TYPE} eq "POINTER" and
 	    $pl->{POINTER_TYPE} eq "ref"
 	    and not $l->{IS_ZERO_TERMINATED}) {
 		$self->pidl("if ($ndr->flags & LIBNDR_FLAG_REF_ALLOC) {");
@@ -2567,18 +2590,18 @@ sub AllocateArrayLevel($$$$$$)
 #####################################################################
 # parse a function
 sub ParseFunctionPull($$)
-{ 
+{
 	my($self,$fn) = @_;
 	my $ndr = "ndr";
 
 	# pull function args
-	$self->fn_declare("pull", $fn, "enum ndr_err_code ndr_pull_$fn->{NAME}(struct ndr_pull *$ndr, int flags, struct $fn->{NAME} *r)") or return;
+	$self->fn_declare("pull", $fn, "enum ndr_err_code ndr_pull_$fn->{NAME}(struct ndr_pull *$ndr, ndr_flags_type flags, struct $fn->{NAME} *r)") or return;
 
 	$self->pidl("{");
 	$self->indent;
 
 	# declare any internal pointers we need
-	foreach my $e (@{$fn->{ELEMENTS}}) { 
+	foreach my $e (@{$fn->{ELEMENTS}}) {
 		$self->DeclarePtrVariables($e);
 		$self->DeclareArrayVariables($e, "pull");
 	}
@@ -2619,12 +2642,12 @@ sub ParseFunctionPull($$)
 
 	foreach my $e (@{$fn->{ELEMENTS}}) {
 		next unless (grep(/out/, @{$e->{DIRECTION}}));
-		next unless ($e->{LEVELS}[0]->{TYPE} eq "POINTER" and 
+		next unless ($e->{LEVELS}[0]->{TYPE} eq "POINTER" and
 		             $e->{LEVELS}[0]->{POINTER_TYPE} eq "ref");
-		next if (($e->{LEVELS}[1]->{TYPE} eq "DATA") and 
-				 ($e->{LEVELS}[1]->{DATA_TYPE} eq "string"));
+		next if (($e->{LEVELS}[1]->{TYPE} eq "DATA") and
+				 (Parse::Pidl::Typelist::is_string_type($e->{LEVELS}[1]->{DATA_TYPE})));
 		next if ($e->{LEVELS}[1]->{TYPE} eq "PIPE");
-		next if (($e->{LEVELS}[1]->{TYPE} eq "ARRAY") 
+		next if (($e->{LEVELS}[1]->{TYPE} eq "ARRAY")
 			and   $e->{LEVELS}[1]->{IS_ZERO_TERMINATED});
 
 		if ($e->{LEVELS}[1]->{TYPE} eq "ARRAY") {
@@ -2647,7 +2670,7 @@ sub ParseFunctionPull($$)
 			}
 		} else {
 			$self->pidl("NDR_PULL_ALLOC($ndr, r->out.$e->{NAME});");
-		
+
 			if (grep(/in/, @{$e->{DIRECTION}})) {
 				$self->pidl("*r->out.$e->{NAME} = *r->in.$e->{NAME};");
 			} else {
@@ -2659,22 +2682,22 @@ sub ParseFunctionPull($$)
 	$self->add_deferred();
 	$self->deindent;
 	$self->pidl("}");
-	
+
 	$self->pidl("if (flags & NDR_OUT) {");
 	$self->indent;
 
 	$self->pidl("#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION");
 
-	# This for fuzzers of ndr_pull where the out elements refer to
+	# This is for fuzzers of ndr_pull where the out elements refer to
 	# in elements in size_is or length_is.
 	#
-	# Not actually very harmful but also not useful outsie a fuzzer
+	# Not actually very harmful but also not useful outside a fuzzer
 	foreach my $e (@{$fn->{ELEMENTS}}) {
 		next unless (grep(/in/, @{$e->{DIRECTION}}));
 		next unless ($e->{LEVELS}[0]->{TYPE} eq "POINTER" and
 		             $e->{LEVELS}[0]->{POINTER_TYPE} eq "ref");
 		next if (($e->{LEVELS}[1]->{TYPE} eq "DATA") and
-				 ($e->{LEVELS}[1]->{DATA_TYPE} eq "string"));
+				 (Parse::Pidl::Typelist::is_string_type($e->{LEVELS}[1]->{DATA_TYPE})));
 		next if ($e->{LEVELS}[1]->{TYPE} eq "PIPE");
 		next if ($e->{LEVELS}[1]->{TYPE} eq "ARRAY");
 
@@ -2887,7 +2910,7 @@ sub FunctionTable($$)
 		$self->pidl("\t$ep, ");
 	}
 	my $endpoint_count = $#{$interface->{ENDPOINTS}}+1;
-	
+
 	$self->pidl("};");
 	$self->pidl("");
 
@@ -2950,7 +2973,7 @@ sub HeaderInclude
 
 #####################################################################
 # generate prototypes and defines for the interface definitions
-# FIXME: these prototypes are for the DCE/RPC client functions, not the 
+# FIXME: these prototypes are for the DCE/RPC client functions, not the
 # NDR parser and so do not belong here, technically speaking
 sub HeaderInterface($$$)
 {
@@ -2972,7 +2995,7 @@ sub HeaderInterface($$$)
 
 	if (defined $interface->{PROPERTIES}->{uuid}) {
 		my $name = uc $interface->{NAME};
-		$self->pidl_hdr("#define NDR_$name\_UUID " . 
+		$self->pidl_hdr("#define NDR_$name\_UUID " .
 		Parse::Pidl::Util::make_str(lc($interface->{UUID})));
 
 		$self->pidl_hdr("#define NDR_$name\_VERSION $interface->{VERSION}");
@@ -2997,12 +3020,12 @@ sub HeaderInterface($$$)
 		next if has_property($_, "noopnum");
 		next if grep(/^$_->{NAME}$/,@{$interface->{INHERITED_FUNCTIONS}});
 		my $u_name = uc $_->{NAME};
-	
+
 		my $val = sprintf("0x%02x", $count);
 		if (defined($interface->{BASE})) {
 			$val .= " + NDR_" . uc $interface->{BASE} . "_CALL_COUNT";
 		}
-		
+
 		$self->pidl_hdr("#define NDR_$u_name ($val)");
 
 		$self->pidl_hdr("");
@@ -3037,7 +3060,7 @@ sub ParseTypePushFunction($$$)
 
 	my $args = $typefamily{$e->{TYPE}}->{DECL}->($e, "push", $e->{NAME}, $varname);
 
-	$self->fn_declare("push", $e, "enum ndr_err_code ".TypeFunctionName("ndr_push", $e)."(struct ndr_push *$ndr, int ndr_flags, $args)") or return;
+	$self->fn_declare("push", $e, "enum ndr_err_code ".TypeFunctionName("ndr_push", $e)."(struct ndr_push *$ndr, ndr_flags_type ndr_flags, $args)") or return;
 
 	$self->pidl("{");
 	$self->indent;
@@ -3066,7 +3089,7 @@ sub ParseTypePullFunction($$)
 
 	my $args = $typefamily{$e->{TYPE}}->{DECL}->($e, "pull", $e->{NAME}, $varname);
 
-	$self->fn_declare("pull", $e, "enum ndr_err_code ".TypeFunctionName("ndr_pull", $e)."(struct ndr_pull *$ndr, int ndr_flags, $args)") or return;
+	$self->fn_declare("pull", $e, "enum ndr_err_code ".TypeFunctionName("ndr_pull", $e)."(struct ndr_pull *$ndr, ndr_flags_type ndr_flags, $args)") or return;
 
 	$self->pidl("{");
 	$self->indent;
@@ -3095,7 +3118,7 @@ sub ParseTypePrintFunction($$$)
 
 	if (is_public_struct($e)) {
                 $self->pidl("static void ".TypeFunctionName("ndr_print_flags", $e).
-                             "(struct ndr_print *$ndr, const char *name, int unused, $args)"
+                             "(struct ndr_print *$ndr, const char *name, ndr_flags_type unused, $args)"
                              );
 		$self->pidl("{");
 		$self->indent;
@@ -3170,8 +3193,8 @@ sub ParseInterface($$$)
 		($needed->{TypeFunctionName("ndr_print", $d)}) && $self->ParseTypePrintFunction($d, "r");
 
 		# Make sure we don't generate a function twice...
-		$needed->{TypeFunctionName("ndr_push", $d)} = 
-		    $needed->{TypeFunctionName("ndr_pull", $d)} = 
+		$needed->{TypeFunctionName("ndr_push", $d)} =
+		    $needed->{TypeFunctionName("ndr_pull", $d)} =
 			$needed->{TypeFunctionName("ndr_print", $d)} = 0;
 
 		($needed->{"ndr_size_$d->{NAME}"}) && $self->ParseTypeNdrSize($d);
@@ -3202,7 +3225,7 @@ sub GenerateIncludes($)
 {
 	my ($self) = @_;
 	if (is_intree()) {
-		$self->pidl("#include \"includes.h\"");
+		$self->pidl("#include \"replace.h\"");
 	} else {
 		$self->pidl("#ifndef _GNU_SOURCE");
 		$self->pidl("#define _GNU_SOURCE");
@@ -3255,7 +3278,7 @@ sub NeededElement($$$)
 
 	return if ($e->{TYPE} eq "EMPTY");
 
-	return if (ref($e->{TYPE}) eq "HASH" and 
+	return if (ref($e->{TYPE}) eq "HASH" and
 		       not defined($e->{TYPE}->{NAME}));
 
 	my ($t, $rt);
@@ -3319,7 +3342,7 @@ sub NeededType($$$)
 		return unless defined($t->{ELEMENTS});
 		for my $e (@{$t->{ELEMENTS}}) {
 			$e->{PARENT} = $t;
-			if (has_property($e, "compression")) { 
+			if (has_property($e, "compression")) {
 				$needed->{"compression"} = 1;
 			}
 			NeededElement($e, $req, $needed);
@@ -3337,7 +3360,7 @@ sub NeededInterface($$)
 	foreach (reverse @{$interface->{TYPES}}) {
 
 		if (has_property($_, "public")) {
-			$needed->{TypeFunctionName("ndr_pull", $_)} = $needed->{TypeFunctionName("ndr_push", $_)} = 
+			$needed->{TypeFunctionName("ndr_pull", $_)} = $needed->{TypeFunctionName("ndr_push", $_)} =
 				$needed->{TypeFunctionName("ndr_print", $_)} = 1;
 		}
 
@@ -3354,7 +3377,7 @@ sub TypeFunctionName($$)
 {
 	my ($prefix, $t) = @_;
 
-	return "$prefix\_$t->{NAME}" if (ref($t) eq "HASH" and 
+	return "$prefix\_$t->{NAME}" if (ref($t) eq "HASH" and
 			$t->{TYPE} eq "TYPEDEF");
 	return "$prefix\_$t->{TYPE}_$t->{NAME}" if (ref($t) eq "HASH");
 	return "$prefix\_$t";
