@@ -17,57 +17,60 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <Python.h>
+#include "lib/replace/system/python.h"
+#include "pyerrors.h"
 #include "python/py3compat.h"
 #include "includes.h"
+#include "python/modules.h"
 #include "version.h"
 #include "param/pyparam.h"
 #include "lib/socket/netif.h"
 #include "lib/util/debug.h"
+#include "lib/util/util_crypt.h"
 #include "librpc/ndr/ndr_private.h"
-
-void init_glue(void);
-static PyObject *PyExc_NTSTATUSError;
-static PyObject *PyExc_WERRORError;
-static PyObject *PyExc_HRESULTError;
-static PyObject *PyExc_DsExtendedError;
+#include "lib/cmdline/cmdline.h"
+#include "lib/crypto/gkdi.h"
 
 static PyObject *py_generate_random_str(PyObject *self, PyObject *args)
 {
-	int len;
+	Py_ssize_t len;
 	PyObject *ret;
 	char *retstr;
-	if (!PyArg_ParseTuple(args, "i", &len)) {
+
+	if (!PyArg_ParseTuple(args, "n", &len)) {
 		return NULL;
 	}
 	if (len < 0) {
 		PyErr_Format(PyExc_ValueError,
-			     "random string length should be positive, not %d",
+			     "random string length should be positive, not %zd",
 			     len);
 		return NULL;
 	}
 	retstr = generate_random_str(NULL, len);
-	ret = PyUnicode_FromString(retstr);
+	if (retstr == NULL) {
+		return PyErr_NoMemory();
+	}
+	ret = PyUnicode_FromStringAndSize(retstr, len);
 	talloc_free(retstr);
 	return ret;
 }
 
 static PyObject *py_generate_random_password(PyObject *self, PyObject *args)
 {
-	int min, max;
+	Py_ssize_t min, max;
 	PyObject *ret;
 	char *retstr;
-	if (!PyArg_ParseTuple(args, "ii", &min, &max)) {
+
+	if (!PyArg_ParseTuple(args, "nn", &min, &max)) {
 		return NULL;
 	}
 	if (max < 0 || min < 0) {
 		/*
-		 * The real range checks happen in generate_random_password().
-		 * Here we are just checking the values won't overflow into
-		 * numbers when cast to size_t.
+		 * The real range checks happens in generate_random_password().
+		 * Here just filter out any negative numbers.
 		 */
 		PyErr_Format(PyExc_ValueError,
-			     "invalid range: %d - %d",
+			     "invalid range: %zd - %zd",
 			     min, max);
 		return NULL;
 	}
@@ -75,11 +78,11 @@ static PyObject *py_generate_random_password(PyObject *self, PyObject *args)
 	retstr = generate_random_password(NULL, min, max);
 	if (retstr == NULL) {
 		if (errno == EINVAL) {
-			PyErr_Format(PyExc_ValueError,
-				     "invalid range: %d - %d",
-				     min, max);
+			return PyErr_Format(PyExc_ValueError,
+					    "invalid range: %zd - %zd",
+					    min, max);
 		}
-		return NULL;
+		return PyErr_NoMemory();
 	}
 	ret = PyUnicode_FromString(retstr);
 	talloc_free(retstr);
@@ -88,21 +91,21 @@ static PyObject *py_generate_random_password(PyObject *self, PyObject *args)
 
 static PyObject *py_generate_random_machine_password(PyObject *self, PyObject *args)
 {
-	int min, max;
+	Py_ssize_t min, max;
 	PyObject *ret;
 	char *retstr;
-	if (!PyArg_ParseTuple(args, "ii", &min, &max)) {
+
+	if (!PyArg_ParseTuple(args, "nn", &min, &max)) {
 		return NULL;
 	}
 	if (max < 0 || min < 0) {
 		/*
-		 * The real range checks happen in
+		 * The real range checks happens in
 		 * generate_random_machine_password().
-		 * Here we are just checking the values won't overflow into
-		 * numbers when cast to size_t.
+		 * Here we just filter out any negative numbers.
 		 */
 		PyErr_Format(PyExc_ValueError,
-			     "invalid range: %d - %d",
+			     "invalid range: %zd - %zd",
 			     min, max);
 		return NULL;
 	}
@@ -110,11 +113,11 @@ static PyObject *py_generate_random_machine_password(PyObject *self, PyObject *a
 	retstr = generate_random_machine_password(NULL, min, max);
 	if (retstr == NULL) {
 		if (errno == EINVAL) {
-			PyErr_Format(PyExc_ValueError,
-				     "invalid range: %d - %d",
-				     min, max);
+			return PyErr_Format(PyExc_ValueError,
+					    "invalid range: %zd - %zd",
+					    min, max);
 		}
-		return NULL;
+		return PyErr_NoMemory();
 	}
 	ret = PyUnicode_FromString(retstr);
 	talloc_free(retstr);
@@ -134,16 +137,16 @@ static PyObject *py_check_password_quality(PyObject *self, PyObject *args)
 
 static PyObject *py_generate_random_bytes(PyObject *self, PyObject *args)
 {
-	int len;
+	Py_ssize_t len;
 	PyObject *ret;
 	uint8_t *bytes = NULL;
 
-	if (!PyArg_ParseTuple(args, "i", &len)) {
+	if (!PyArg_ParseTuple(args, "n", &len)) {
 		return NULL;
 	}
 	if (len < 0) {
 		PyErr_Format(PyExc_ValueError,
-			     "random bytes length should be positive, not %d",
+			     "random bytes length should be positive, not %zd",
 			     len);
 		return NULL;
 	}
@@ -308,6 +311,16 @@ static PyObject *py_is_ad_dc_built(PyObject *self,
 #endif
 }
 
+static PyObject *py_is_rust_built(PyObject *self,
+		PyObject *Py_UNUSED(ignored))
+{
+#ifdef HAVE_RUST
+	Py_RETURN_TRUE;
+#else
+	Py_RETURN_FALSE;
+#endif
+}
+
 static PyObject *py_is_selftest_enabled(PyObject *self,
                 PyObject *Py_UNUSED(ignored))
 {
@@ -353,7 +366,7 @@ static PyObject *py_interface_ips(PyObject *self, PyObject *args)
 	lp_ctx = lpcfg_from_py_object(tmp_ctx, py_lp_ctx);
 	if (lp_ctx == NULL) {
 		talloc_free(tmp_ctx);
-		return NULL;
+		return PyErr_NoMemory();
 	}
 
 	load_interface_list(tmp_ctx, lp_ctx, &ifaces);
@@ -423,8 +436,8 @@ static PyObject *py_interface_ips(PyObject *self, PyObject *args)
 
 static PyObject *py_strcasecmp_m(PyObject *self, PyObject *args)
 {
-	const char *s1 = NULL;
-	const char *s2 = NULL;
+	char *s1 = NULL;
+	char *s2 = NULL;
 	long cmp_result = 0;
 	if (!PyArg_ParseTuple(args, PYARG_STR_UNI
 			      PYARG_STR_UNI,
@@ -433,15 +446,15 @@ static PyObject *py_strcasecmp_m(PyObject *self, PyObject *args)
 	}
 
 	cmp_result = strcasecmp_m(s1, s2);
-	PyMem_Free(discard_const_p(char, s1));
-	PyMem_Free(discard_const_p(char, s2));
+	PyMem_Free(s1);
+	PyMem_Free(s2);
 	return PyLong_FromLong(cmp_result);
 }
 
 static PyObject *py_strstr_m(PyObject *self, PyObject *args)
 {
-	const char *s1 = NULL;
-	const char *s2 = NULL;
+	char *s1 = NULL;
+	char *s2 = NULL;
 	char *strstr_ret = NULL;
 	PyObject *result = NULL;
 	if (!PyArg_ParseTuple(args, PYARG_STR_UNI
@@ -451,16 +464,109 @@ static PyObject *py_strstr_m(PyObject *self, PyObject *args)
 
 	strstr_ret = strstr_m(s1, s2);
 	if (!strstr_ret) {
-		PyMem_Free(discard_const_p(char, s1));
-		PyMem_Free(discard_const_p(char, s2));
+		PyMem_Free(s1);
+		PyMem_Free(s2);
 		Py_RETURN_NONE;
 	}
 	result = PyUnicode_FromString(strstr_ret);
-	PyMem_Free(discard_const_p(char, s1));
-	PyMem_Free(discard_const_p(char, s2));
+	PyMem_Free(s1);
+	PyMem_Free(s2);
 	return result;
 }
 
+static PyObject *py_get_burnt_commandline(PyObject *self, PyObject *args)
+{
+	PyObject *cmdline_as_list, *ret;
+	char *burnt_cmdline = NULL;
+	Py_ssize_t i, argc;
+	char **argv = NULL;
+	TALLOC_CTX *frame = talloc_stackframe();
+	bool burnt;
+
+	if (!PyArg_ParseTuple(args, "O!", &PyList_Type, &cmdline_as_list))
+	{
+		TALLOC_FREE(frame);
+		return NULL;
+	}
+
+	argc = PyList_GET_SIZE(cmdline_as_list);
+
+	if (argc == 0) {
+		TALLOC_FREE(frame);
+		Py_RETURN_NONE;
+	}
+
+	argv = PyList_AsStringList(frame, cmdline_as_list, "sys.argv");
+	if (argv == NULL) {
+		TALLOC_FREE(frame);
+		return NULL;
+	}
+
+	burnt = samba_cmdline_burn(argc, argv);
+	if (!burnt) {
+		TALLOC_FREE(frame);
+		Py_RETURN_NONE;
+	}
+
+	for (i = 0; i < argc; i++) {
+		if (i == 0) {
+			burnt_cmdline = talloc_strdup(frame,
+						      argv[i]);
+		} else {
+			burnt_cmdline
+				= talloc_asprintf_append(burnt_cmdline,
+							 " %s",
+							 argv[i]);
+		}
+		if (burnt_cmdline == NULL) {
+			PyErr_NoMemory();
+			TALLOC_FREE(frame);
+			return NULL;
+		}
+	}
+
+	ret = PyUnicode_FromString(burnt_cmdline);
+	TALLOC_FREE(frame);
+
+	return ret;
+}
+
+static PyObject *py_crypt(PyObject *self, PyObject *args)
+{
+	PyObject *py_hash = NULL;
+	char *phrase = NULL;
+	char *setting = NULL;
+	TALLOC_CTX *frame = NULL;
+	int ret;
+	DATA_BLOB hash = {};
+
+	if (!PyArg_ParseTuple(args, "ss", &phrase, &setting)) {
+		TALLOC_FREE(frame);
+		return NULL;
+	}
+	frame = talloc_stackframe();
+	ret = talloc_crypt_blob(frame, phrase, setting, &hash);
+	if (ret != 0) {
+		const char *errstr = talloc_crypt_errstring(frame, ret);
+		if (ret == EINVAL || ret == ERANGE || ret == ENOTRECOVERABLE) {
+			PyErr_Format(PyExc_ValueError,
+				     "could not crypt(): %s",
+				     errstr);
+		} else {
+			PyErr_Format(PyExc_OSError,
+				     "could not crypt(): %s",
+				     errstr);
+		}
+		TALLOC_FREE(frame);
+		return NULL;
+	}
+
+	py_hash = PyUnicode_FromStringAndSize((char *)hash.data, hash.length);
+	TALLOC_FREE(frame);
+	return py_hash;
+}
+
+
 static PyMethodDef py_misc_methods[] = {
 	{ "generate_random_str", (PyCFunction)py_generate_random_str, METH_VARARGS,
 		"generate_random_str(len) -> string\n"
@@ -473,12 +579,12 @@ static PyMethodDef py_misc_methods[] = {
 		METH_VARARGS, "generate_random_machine_password(min, max) -> string\n"
 		"Generate random password "
 		"(based on random utf16 characters converted to utf8 or "
-		"random ascii characters if 'unix charset' is not 'utf8')"
+		"random ascii characters if 'unix charset' is not 'utf8') "
 		"with a length >= min (at least 14) and <= max (at most 255)." },
 	{ "check_password_quality", (PyCFunction)py_check_password_quality,
 		METH_VARARGS, "check_password_quality(pass) -> bool\n"
-		"Check password quality against Samba's check_password_quality,"
-		"the implementation of Microsoft's rules:"
+		"Check password quality against Samba's check_password_quality, "
+		"the implementation of Microsoft's rules: "
 		"http://msdn.microsoft.com/en-us/subscriptions/cc786468%28v=ws.10%29.aspx"
 	},
 	{ "unix2nttime", (PyCFunction)py_unix2nttime, METH_VARARGS,
@@ -508,7 +614,7 @@ static PyMethodDef py_misc_methods[] = {
 	{ "is_ntvfs_fileserver_built", (PyCFunction)py_is_ntvfs_fileserver_built, METH_NOARGS,
 		"is the NTVFS file server built in this installation?" },
 	{ "is_heimdal_built", (PyCFunction)py_is_heimdal_built, METH_NOARGS,
-		"is Samba built with Heimdal Kerberbos?" },
+		"is Samba built with Heimdal Kerberos?" },
 	{ "generate_random_bytes",
 		(PyCFunction)py_generate_random_bytes,
 		METH_VARARGS,
@@ -520,6 +626,13 @@ static PyMethodDef py_misc_methods[] = {
 		METH_NOARGS, "is Samba built with selftest enabled?" },
 	{ "ndr_token_max_list_size", (PyCFunction)py_ndr_token_max_list_size,
 		METH_NOARGS, "How many NDR internal tokens is too many for this build?" },
+	{ "get_burnt_commandline", (PyCFunction)py_get_burnt_commandline,
+		METH_VARARGS, "Return a redacted commandline to feed to setproctitle (None if no redaction required)" },
+	{ "crypt", (PyCFunction)py_crypt,
+		METH_VARARGS,
+		"encrypt as phrase, per crypt(3), as determined by setting." },
+	{ "is_rust_built", (PyCFunction)py_is_rust_built, METH_NOARGS,
+		"is Samba built with Rust?" },
 	{0}
 };
 
@@ -534,6 +647,8 @@ static struct PyModuleDef moduledef = {
 MODULE_INIT_FUNC(_glue)
 {
 	PyObject *m;
+	PyObject *py_obj = NULL;
+	int ret;
 
 	debug_setup_talloc_log();
 
@@ -543,28 +658,57 @@ MODULE_INIT_FUNC(_glue)
 
 	PyModule_AddObject(m, "version",
 					   PyUnicode_FromString(SAMBA_VERSION_STRING));
-	PyExc_NTSTATUSError = PyErr_NewException(discard_const_p(char, "samba.NTSTATUSError"), PyExc_RuntimeError, NULL);
-	if (PyExc_NTSTATUSError != NULL) {
-		Py_INCREF(PyExc_NTSTATUSError);
-		PyModule_AddObject(m, "NTSTATUSError", PyExc_NTSTATUSError);
+	py_obj = PyErr_NewException("samba.NTSTATUSError", PyExc_RuntimeError, NULL);
+	if (py_obj != NULL) {
+		PyModule_AddObject(m, "NTSTATUSError", py_obj);
 	}
 
-	PyExc_WERRORError = PyErr_NewException(discard_const_p(char, "samba.WERRORError"), PyExc_RuntimeError, NULL);
-	if (PyExc_WERRORError != NULL) {
-		Py_INCREF(PyExc_WERRORError);
-		PyModule_AddObject(m, "WERRORError", PyExc_WERRORError);
+	py_obj = PyErr_NewException("samba.WERRORError", PyExc_RuntimeError, NULL);
+	if (py_obj != NULL) {
+		PyModule_AddObject(m, "WERRORError", py_obj);
 	}
 
-	PyExc_HRESULTError = PyErr_NewException(discard_const_p(char, "samba.HRESULTError"), PyExc_RuntimeError, NULL);
-	if (PyExc_HRESULTError != NULL) {
-		Py_INCREF(PyExc_HRESULTError);
-		PyModule_AddObject(m, "HRESULTError", PyExc_HRESULTError);
+	py_obj = PyErr_NewException("samba.HRESULTError", PyExc_RuntimeError, NULL);
+	if (py_obj != NULL) {
+		PyModule_AddObject(m, "HRESULTError", py_obj);
 	}
 
-	PyExc_DsExtendedError = PyErr_NewException(discard_const_p(char, "samba.DsExtendedError"), PyExc_RuntimeError, NULL);
-	if (PyExc_DsExtendedError != NULL) {
-		Py_INCREF(PyExc_DsExtendedError);
-		PyModule_AddObject(m, "DsExtendedError", PyExc_DsExtendedError);
+	py_obj = PyErr_NewException("samba.DsExtendedError", PyExc_RuntimeError, NULL);
+	if (py_obj != NULL) {
+		PyModule_AddObject(m, "DsExtendedError", py_obj);
+	}
+
+	ret = PyModule_AddIntConstant(m, "GKDI_L1_KEY_ITERATION", gkdi_l1_key_iteration);
+	if (ret) {
+		Py_DECREF(m);
+		return NULL;
+	}
+	ret = PyModule_AddIntConstant(m, "GKDI_L2_KEY_ITERATION", gkdi_l2_key_iteration);
+	if (ret) {
+		Py_DECREF(m);
+		return NULL;
+	}
+	py_obj = PyLong_FromLongLong(gkdi_key_cycle_duration);
+	if (py_obj == NULL) {
+		Py_DECREF(m);
+		return NULL;
+	}
+	ret = PyModule_AddObject(m, "GKDI_KEY_CYCLE_DURATION", py_obj);
+	if (ret) {
+		Py_DECREF(py_obj);
+		Py_DECREF(m);
+		return NULL;
+	}
+	py_obj = PyLong_FromLongLong(gkdi_max_clock_skew);
+	if (py_obj == NULL) {
+		Py_DECREF(m);
+		return NULL;
+	}
+	ret = PyModule_AddObject(m, "GKDI_MAX_CLOCK_SKEW", py_obj);
+	if (ret) {
+		Py_DECREF(py_obj);
+		Py_DECREF(m);
+		return NULL;
 	}
 
 	return m;
