@@ -40,7 +40,7 @@ class AuditLogDsdbTests(AuditLogTestBase):
     def setUp(self):
         self.message_type = MSG_DSDB_LOG
         self.event_type = DSDB_EVENT_NAME
-        super(AuditLogDsdbTests, self).setUp()
+        super().setUp()
 
         self.server_ip = os.environ["SERVER_IP"]
 
@@ -92,7 +92,7 @@ class AuditLogDsdbTests(AuditLogTestBase):
 
     def tearDown(self):
         self.discardMessages()
-        super(AuditLogDsdbTests, self).tearDown()
+        super().tearDown()
 
     def haveExpectedTxn(self, expected):
         if self.context["txnMessage"] is not None:
@@ -144,8 +144,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Modify", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertTrue(dn.lower(), audit["dn"].lower())
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
         # We skip the check for self.get_service_description() as this
@@ -185,8 +185,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Modify", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertEqual(dn, audit["dn"])
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
         # We skip the check for self.get_service_description() as this
@@ -227,8 +227,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Modify", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertEqual(dn, audit["dn"])
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertTrue(self.is_guid(audit["sessionId"]))
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
@@ -266,8 +266,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Modify", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertTrue(dn.lower(), audit["dn"].lower())
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertTrue(self.is_guid(audit["sessionId"]))
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
@@ -297,8 +297,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Add", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertEqual(dn, audit["dn"])
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
         service_description = self.get_service_description()
@@ -343,8 +343,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Delete", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertTrue(dn.lower(), audit["dn"].lower())
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertTrue(self.is_guid(audit["sessionId"]))
         self.assertEqual(0, audit["statusCode"])
         self.assertEqual("Success", audit["status"])
@@ -383,8 +383,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Delete", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertTrue(dn.lower(), audit["dn"].lower())
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertEqual(ERR_NO_SUCH_OBJECT, audit["statusCode"])
         self.assertEqual("No such object", audit["status"])
         self.assertTrue(self.is_guid(audit["sessionId"]))
@@ -431,8 +431,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Add", audit["operation"])
         self.assertTrue(audit["performedAsSystem"])
         self.assertTrue(dn.lower(), audit["dn"].lower())
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertTrue(self.is_guid(audit["sessionId"]))
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
@@ -480,8 +480,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Delete", audit["operation"])
         self.assertTrue(audit["performedAsSystem"])
         self.assertTrue(dn.lower(), audit["dn"].lower())
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertTrue(self.is_guid(audit["sessionId"]))
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
@@ -514,8 +514,8 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.assertEqual("Modify", audit["operation"])
         self.assertFalse(audit["performedAsSystem"])
         self.assertEqual(dn, audit["dn"])
-        self.assertRegexpMatches(audit["remoteAddress"],
-                                 self.remoteAddress)
+        self.assertRegex(audit["remoteAddress"],
+                         self.remoteAddress)
         self.assertTrue(self.is_guid(audit["sessionId"]))
         session_id = self.get_session()
         self.assertEqual(session_id, audit["sessionId"])
@@ -587,7 +587,7 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.discardMessages()
         self.ldb.modify_ldif(
             "dn: " + dn + "\n" +
-            "changetype: delete\n" +
+            "changetype: modify\n" +
             "delete: carLicense\n" +
             "carLicense: license-03\n" +
             "carLicense: license-04\n")
@@ -613,7 +613,7 @@ class AuditLogDsdbTests(AuditLogTestBase):
         self.discardMessages()
         self.ldb.modify_ldif(
             "dn: " + dn + "\n" +
-            "changetype: delete\n" +
+            "changetype: modify\n" +
             "replace: carLicense\n" +
             "carLicense: license-05\n" +
             "carLicense: license-06\n")
