@@ -25,13 +25,13 @@ class ForestCmdTestCase(SambaToolCmdTest):
     samdb = None
 
     def setUp(self):
-        super(ForestCmdTestCase, self).setUp()
+        super().setUp()
         self.samdb = self.getSamDB("-H", "ldap://%s" % os.environ["DC_SERVER"],
                                    "-U%s%%%s" % (os.environ["DC_USERNAME"], os.environ["DC_PASSWORD"]))
         self.domain_dn = self.samdb.domain_dn()
 
     def tearDown(self):
-        super(ForestCmdTestCase, self).tearDown()
+        super().tearDown()
         # Reset the values we might have changed.
         ds_dn = "CN=Directory Service,CN=Windows NT,CN=Services,CN=Configuration"
         m = ldb.Message()
@@ -43,25 +43,28 @@ class ForestCmdTestCase(SambaToolCmdTest):
 
     def test_display(self):
         """Tests that we can display forest settings"""
-        (result, out, err) = self.runsublevelcmd("forest", ("directory_service",
-                                                            "show"),
-                                                 "-H", "ldap://%s" % os.environ["DC_SERVER"],
-                                                 "-U%s%%%s" % (os.environ["DC_USERNAME"],
-                                                               os.environ["DC_PASSWORD"]))
+        (result, out, err) = self.runcmd("forest",
+                                         "directory_service",
+                                         "show",
+                                         "-H", "ldap://%s" % os.environ["DC_SERVER"],
+                                         "-U%s%%%s" % (os.environ["DC_USERNAME"],
+                                                       os.environ["DC_PASSWORD"]))
 
         self.assertCmdSuccess(result, out, err)
-        self.assertEquals(err, "", "Shouldn't be any error messages")
+        self.assertEqual(err, "", "Shouldn't be any error messages")
         self.assertIn("dsheuristics: <NO VALUE>", out)
 
     def test_modify_dsheuristics(self):
         """Test that we can modify the dsheuristics setting"""
 
-        (result, out, err) = self.runsublevelcmd("forest", ("directory_service",
-                                                            "dsheuristics"), "0000002",
-                                                 "-H", "ldap://%s" % os.environ["DC_SERVER"],
-                                                 "-U%s%%%s" % (os.environ["DC_USERNAME"],
-                                                               os.environ["DC_PASSWORD"]))
+        (result, out, err) = self.runcmd("forest",
+                                         "directory_service",
+                                         "dsheuristics",
+                                         "0000002",
+                                         "-H", "ldap://%s" % os.environ["DC_SERVER"],
+                                         "-U%s%%%s" % (os.environ["DC_USERNAME"],
+                                                       os.environ["DC_PASSWORD"]))
 
         self.assertCmdSuccess(result, out, err)
-        self.assertEquals(err, "", "Shouldn't be any error messages")
+        self.assertEqual(err, "", "Shouldn't be any error messages")
         self.assertIn("set dsheuristics: 0000002", out)
