@@ -817,10 +817,8 @@ err:
  * End of data: return NULL
  * Failure: set errno, return NULL
  */
-static struct dirent *mh_readdir(vfs_handle_struct *handle,
-				 struct files_struct *dirfsp,
-				 DIR *dirp,
-				 SMB_STRUCT_STAT *sbuf)
+static struct dirent *
+mh_readdir(vfs_handle_struct *handle, struct files_struct *dirfsp, DIR *dirp)
 {
 	mh_dirinfo_struct* dirInfo = (mh_dirinfo_struct*)dirp;
 	struct dirent *d = NULL;
@@ -843,7 +841,7 @@ static struct dirent *mh_readdir(vfs_handle_struct *handle,
 
 	if (! dirInfo->isInMediaFiles)
 	{
-		d = SMB_VFS_NEXT_READDIR(handle, dirfsp, dirInfo->dirstream, sbuf);
+		d = SMB_VFS_NEXT_READDIR(handle, dirfsp, dirInfo->dirstream);
 		goto out;
 	}
 
@@ -853,7 +851,7 @@ static struct dirent *mh_readdir(vfs_handle_struct *handle,
 		bool isAppleDouble;
 
 		skip = False;
-		d = SMB_VFS_NEXT_READDIR(handle, dirfsp, dirInfo->dirstream, sbuf);
+		d = SMB_VFS_NEXT_READDIR(handle, dirfsp, dirInfo->dirstream);
 
 		if (d == NULL)
 		{
@@ -948,31 +946,6 @@ out:
 	return d;
 }
 
-/*
- * Success: no success result defined.
- * Failure: no failure result defined.
- */
-static void mh_seekdir(vfs_handle_struct *handle,
-		DIR *dirp,
-		long offset)
-{
-	DEBUG(MH_INFO_DEBUG, ("Entering and leaving mh_seekdir\n"));
-	SMB_VFS_NEXT_SEEKDIR(handle,
-			((mh_dirinfo_struct*)dirp)->dirstream, offset);
-}
-
-/*
- * Success: return long
- * Failure: no failure result defined.
- */
-static long mh_telldir(vfs_handle_struct *handle,
-		DIR *dirp)
-{
-	DEBUG(MH_INFO_DEBUG, ("Entering and leaving mh_telldir\n"));
-	return SMB_VFS_NEXT_TELLDIR(handle,
-			((mh_dirinfo_struct*)dirp)->dirstream);
-}
-
 /*
  * Success: no success result defined.
  * Failure: no failure result defined.
@@ -1222,7 +1195,8 @@ static int mh_renameat(vfs_handle_struct *handle,
 		files_struct *srcfsp,
 		const struct smb_filename *smb_fname_src,
 		files_struct *dstfsp,
-		const struct smb_filename *smb_fname_dst)
+		const struct smb_filename *smb_fname_dst,
+		const struct vfs_rename_how *how)
 {
 	int status = -1;
 	struct smb_filename *full_fname_src = NULL;
@@ -1244,7 +1218,8 @@ static int mh_renameat(vfs_handle_struct *handle,
 				srcfsp,
 				smb_fname_src,
 				dstfsp,
-				smb_fname_dst);
+				smb_fname_dst,
+				how);
 		goto out;
 	}
 
@@ -1283,7 +1258,8 @@ static int mh_renameat(vfs_handle_struct *handle,
 				srcfsp->conn->cwd_fsp,
 				srcClientFname,
 				dstfsp->conn->cwd_fsp,
-				dstClientFname);
+				dstClientFname,
+				how);
 err:
 	TALLOC_FREE(full_fname_src);
 	TALLOC_FREE(full_fname_dst);
@@ -1845,8 +1821,6 @@ static struct vfs_fn_pointers vfs_mh_fns = {
 
 	.fdopendir_fn = mh_fdopendir,
 	.readdir_fn = mh_readdir,
-	.seekdir_fn = mh_seekdir,
-	.telldir_fn = mh_telldir,
 	.rewind_dir_fn = mh_rewinddir,
 	.mkdirat_fn = mh_mkdirat,
 	.closedir_fn = mh_closedir,
