@@ -726,31 +726,42 @@ static bool _shadow_copy2_strip_snapshot_converted(TALLOC_CTX *mem_ctx,
 static char *shadow_copy2_find_mount_point(TALLOC_CTX *mem_ctx,
 					   vfs_handle_struct *handle)
 {
-	char *path = talloc_strdup(mem_ctx, handle->conn->connectpath);
+	struct smb_filename *smb_fname_cpath = NULL;
 	dev_t dev;
-	struct stat st;
 	char *p;
 
-	if (stat(path, &st) != 0) {
-		talloc_free(path);
+	smb_fname_cpath = synthetic_smb_fname(mem_ctx,
+					      handle->conn->connectpath,
+					      NULL,
+					      NULL,
+					      0,
+					      0);
+	if (smb_fname_cpath == NULL) {
+		errno = ENOMEM;
 		return NULL;
 	}
 
-	dev = st.st_dev;
+	if (SMB_VFS_NEXT_STAT(handle, smb_fname_cpath) != 0) {
+		TALLOC_FREE(smb_fname_cpath);
+		return NULL;
+	}
 
-	while ((p = strrchr(path, '/')) && p > path) {
+	dev = smb_fname_cpath->st.st_ex_dev;
+
+	while ((p = strrchr(smb_fname_cpath->base_name, '/')) &&
+			p > smb_fname_cpath->base_name) {
 		*p = 0;
-		if (stat(path, &st) != 0) {
-			talloc_free(path);
+		if (SMB_VFS_NEXT_STAT(handle, smb_fname_cpath) != 0) {
+			TALLOC_FREE(smb_fname_cpath);
 			return NULL;
 		}
-		if (st.st_dev != dev) {
+		if (smb_fname_cpath->st.st_ex_dev != dev) {
 			*p = '/';
 			break;
 		}
 	}
 
-	return path;
+	return smb_fname_cpath->base_name;
 }
 
 /**
@@ -1027,7 +1038,8 @@ static int shadow_copy2_renameat(vfs_handle_struct *handle,
 				files_struct *srcfsp,
 				const struct smb_filename *smb_fname_src,
 				files_struct *dstfsp,
-				const struct smb_filename *smb_fname_dst)
+				const struct smb_filename *smb_fname_dst,
+				const struct vfs_rename_how *how)
 {
 	time_t timestamp_src = 0;
 	time_t timestamp_dst = 0;
@@ -1069,7 +1081,8 @@ static int shadow_copy2_renameat(vfs_handle_struct *handle,
 			srcfsp,
 			smb_fname_src,
 			dstfsp,
-			smb_fname_dst);
+			smb_fname_dst,
+			how);
 }
 
 static int shadow_copy2_symlinkat(vfs_handle_struct *handle,
@@ -1519,25 +1532,15 @@ static struct smb_filename *shadow_copy2_openat_name(
 {
 	struct smb_filename *result = NULL;
 
-	if (fsp->base_fsp != NULL) {
+	if (fsp_is_alternate_stream(fsp)) {
 		struct smb_filename *base_fname = fsp->base_fsp->fsp_name;
 
-		if (smb_fname_in->base_name[0] == '/') {
-			/*
-			 * Special-case stream names from streams_depot
-			 */
-			result = cp_smb_filename(mem_ctx, smb_fname_in);
-		} else {
-
-			SMB_ASSERT(is_named_stream(smb_fname_in));
-
-			result = synthetic_smb_fname(mem_ctx,
-						     base_fname->base_name,
-						     smb_fname_in->stream_name,
-						     &smb_fname_in->st,
-						     smb_fname_in->twrp,
-						     smb_fname_in->flags);
-		}
+		result = synthetic_smb_fname(mem_ctx,
+					     base_fname->base_name,
+					     smb_fname_in->stream_name,
+					     &smb_fname_in->st,
+					     smb_fname_in->twrp,
+					     smb_fname_in->flags);
 	} else {
 		result = full_path_from_dirfsp_atname(
 			mem_ctx, dirfsp, smb_fname_in);
