@@ -897,7 +897,7 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
 			return 0;
 		}
 
-		/* need to continue the the bottom of the function to
+		/* need to continue at the bottom of the function to
 		   save the correct attributes */
 	}
 
@@ -967,10 +967,12 @@ static int traverse_fn_delete(TDB_CONTEXT *t, TDB_DATA key, TDB_DATA data, void
 		   Don't delete the job if it was submitted after the lpq_time. */
 
 		if (pjob.starttime < ts->lpq_time) {
-			DEBUG(10,("traverse_fn_delete: pjob %u deleted due to pjob.starttime (%u) < ts->lpq_time (%u)\n",
-						(unsigned int)jobid,
-						(unsigned int)pjob.starttime,
-						(unsigned int)ts->lpq_time ));
+			DBG_DEBUG("pjob %u deleted due to pjob.starttime "
+				  "(%" PRIu64 ") < ts->lpq_time (%" PRIu64
+				  ")\n",
+				  (unsigned int)jobid,
+				  (uint64_t)pjob.starttime,
+				  (uint64_t)ts->lpq_time);
 			pjob_delete(ts->ev, ts->msg_ctx,
 				    ts->sharename, jobid);
 		} else
@@ -1229,10 +1231,13 @@ static bool print_cache_expired(const char *sharename, bool check_pending)
 		uint32_t u;
 		time_t msg_pending_time;
 
-		DEBUG(4, ("print_cache_expired: cache expired for queue %s "
-			"(last_qscan_time = %d, time now = %d, qcachetime = %d)\n",
-			sharename, (int)last_qscan_time, (int)time_now,
-			(int)lp_lpq_cache_time() ));
+		DBG_INFO("cache expired for queue %s "
+			 "(last_qscan_time = %" PRIu64 ", time now = %" PRIu64
+			 ", qcachetime = %d)\n",
+			 sharename,
+			 (uint64_t)last_qscan_time,
+			 (uint64_t)time_now,
+			 lp_lpq_cache_time());
 
 		/* check if another smbd has already sent a message to update the
 		   queue.  Give the pending message one minute to clear and
@@ -1433,7 +1438,7 @@ static void print_queue_update_internal(struct tevent_context *ev,
 
 /****************************************************************************
  Update the internal database from the system print queue for a queue.
- obtain a lock on the print queue before proceeding (needed when mutiple
+ obtain a lock on the print queue before proceeding (needed when multiple
  smbd processes maytry to update the lpq cache concurrently).
 ****************************************************************************/
 
@@ -1592,7 +1597,7 @@ static void print_queue_update(struct messaging_context *msg_ctx,
 			"",
 			get_current_gid(NULL),
 			get_current_username(),
-			current_user_info.domain,
+			get_current_user_info_domain(),
 			lpqcommand);
 	if (!lpqcommand) {
 		return;
@@ -1612,7 +1617,7 @@ static void print_queue_update(struct messaging_context *msg_ctx,
 			"",
 			get_current_gid(NULL),
 			get_current_username(),
-			current_user_info.domain,
+			get_current_user_info_domain(),
 			lprmcommand);
 	if (!lprmcommand) {
 		return;
@@ -1705,7 +1710,7 @@ bool print_notify_register_pid(int snum)
 
 	/* if (snum == -1), then the change notify request was
 	   on a print server handle and we need to register on
-	   all print queus */
+	   all print queues */
 
 	if (snum == -1)
 	{
@@ -1887,7 +1892,7 @@ bool print_job_exists(const char* sharename, uint32_t jobid)
 }
 
 /****************************************************************************
- Return the device mode asigned to a specific print job.
+ Return the device mode assigned to a specific print job.
  Only valid for the process doing the spooling and when the job
  has not been spooled.
 ****************************************************************************/
@@ -1997,7 +2002,7 @@ static bool print_job_delete1(struct tevent_context *ev,
 
 	/* Hrm - we need to be able to cope with deleting a job before it
 	   has reached the spooler.  Just mark it as LPQ_DELETING and
-	   let the print_queue_update() code rmeove the record */
+	   let the print_queue_update() code remove the record */
 
 
 	if (pjob->sysjob == -1) {
@@ -2096,8 +2101,8 @@ WERROR print_job_delete(const struct auth_session_info *server_info,
 	if (!owner &&
 	    !W_ERROR_IS_OK(print_access_check(server_info, msg_ctx, snum,
 					      JOB_ACCESS_ADMINISTER))) {
-		DEBUG(0, ("print job delete denied."
-			  "User name: %s, Printer name: %s.",
+		DEBUG(0, ("print job delete denied. "
+			  "User name: %s, Printer name: %s.\n",
 			  uidtoname(server_info->unix_token->uid),
 			  lp_printername(tmp_ctx, lp_sub, snum)));
 
@@ -2183,8 +2188,8 @@ WERROR print_job_pause(const struct auth_session_info *server_info,
 	if (!is_owner(server_info, lp_const_servicename(snum), jobid) &&
 	    !W_ERROR_IS_OK(print_access_check(server_info, msg_ctx, snum,
 					      JOB_ACCESS_ADMINISTER))) {
-		DEBUG(0, ("print job pause denied."
-			  "User name: %s, Printer name: %s.",
+		DEBUG(0, ("print job pause denied. "
+			  "User name: %s, Printer name: %s.\n",
 			  uidtoname(server_info->unix_token->uid),
 			  lp_printername(tmp_ctx, lp_sub, snum)));
 
@@ -2252,8 +2257,8 @@ WERROR print_job_resume(const struct auth_session_info *server_info,
 	if (!is_owner(server_info, lp_const_servicename(snum), jobid) &&
 	    !W_ERROR_IS_OK(print_access_check(server_info, msg_ctx, snum,
 					      JOB_ACCESS_ADMINISTER))) {
-		DEBUG(0, ("print job resume denied."
-			  "User name: %s, Printer name: %s.",
+		DEBUG(0, ("print job resume denied. "
+			  "User name: %s, Printer name: %s.\n",
 			  uidtoname(server_info->unix_token->uid),
 			  lp_printername(tmp_ctx, lp_sub, snum)));
 
@@ -2462,10 +2467,10 @@ static WERROR allocate_print_jobid(struct tdb_print_db *pdb, int snum,
 	/* Store a dummy placeholder. */
 	{
 		uint32_t tmp;
-		TDB_DATA dum;
-		dum.dptr = NULL;
-		dum.dsize = 0;
-		if (tdb_store(pdb->tdb, print_key(jobid, &tmp), dum,
+		TDB_DATA dummy = {
+			.dsize = 0,
+		};
+		if (tdb_store(pdb->tdb, print_key(jobid, &tmp), dummy,
 			      TDB_INSERT) != 0) {
 			DEBUG(3, ("allocate_print_jobid: "
 				  "jobid (%d) failed to store placeholder.\n",
@@ -2850,7 +2855,7 @@ NTSTATUS print_job_end(struct messaging_context *msg_ctx, int snum,
 				      "",
 				      get_current_gid(NULL),
 				      get_current_username(),
-				      current_user_info.domain,
+				      get_current_user_info_domain(),
 				      lpq_cmd);
 	if (lpq_cmd == NULL) {
 		status = NT_STATUS_PRINT_CANCELLED;
