@@ -186,7 +186,7 @@ static NTSTATUS cmd_lsa_query_info_policy(struct rpc_pipe_client *cli,
 		uint32_t out_version = 0;
 
 		status = dcerpc_lsa_open_policy_fallback(
-			b,
+			cli,
 			mem_ctx,
 			cli->srv_name_slash,
 			true,
@@ -938,7 +938,7 @@ static NTSTATUS cmd_lsa_create_account(struct rpc_pipe_client *cli,
 	if (!NT_STATUS_IS_OK(status))
 		goto done;
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1004,7 +1004,7 @@ static NTSTATUS cmd_lsa_enum_privsaccounts(struct rpc_pipe_client *cli,
 	if (!NT_STATUS_IS_OK(status))
 		goto done;
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1089,7 +1089,7 @@ static NTSTATUS cmd_lsa_enum_acct_rights(struct rpc_pipe_client *cli,
 	if (!NT_STATUS_IS_OK(status))
 		goto done;
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1156,7 +1156,7 @@ static NTSTATUS cmd_lsa_add_acct_rights(struct rpc_pipe_client *cli,
 	if (!NT_STATUS_IS_OK(status))
 		goto done;
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1227,7 +1227,7 @@ static NTSTATUS cmd_lsa_remove_acct_rights(struct rpc_pipe_client *cli,
 	if (!NT_STATUS_IS_OK(status))
 		goto done;
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1295,7 +1295,7 @@ static NTSTATUS cmd_lsa_lookup_priv_value(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1358,7 +1358,7 @@ static NTSTATUS cmd_lsa_query_secobj(struct rpc_pipe_client *cli,
 	if (argc == 2)
 		sscanf(argv[1], "%x", &sec_info);
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1463,7 +1463,7 @@ static NTSTATUS cmd_lsa_query_trustdominfobysid(struct rpc_pipe_client *cli,
 	if (argc == 3)
 		info_class = atoi(argv[2]);
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1489,7 +1489,8 @@ static NTSTATUS cmd_lsa_query_trustdominfobysid(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0, ("Could not retrieve session key: %s\n", nt_errstr(status)));
 		goto done;
@@ -1531,7 +1532,7 @@ static NTSTATUS cmd_lsa_query_trustdominfobyname(struct rpc_pipe_client *cli,
 	if (argc == 3)
 		info_class = atoi(argv[2]);
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1559,7 +1560,8 @@ static NTSTATUS cmd_lsa_query_trustdominfobyname(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0, ("Could not retrieve session key: %s\n", nt_errstr(status)));
 		goto done;
@@ -1611,7 +1613,7 @@ static NTSTATUS cmd_lsa_set_trustdominfo(struct rpc_pipe_client *cli,
 		return NT_STATUS_INVALID_PARAMETER;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1689,7 +1691,7 @@ static NTSTATUS cmd_lsa_query_trustdominfo(struct rpc_pipe_client *cli,
 	if (argc == 3)
 		info_class = atoi(argv[2]);
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1727,7 +1729,8 @@ static NTSTATUS cmd_lsa_query_trustdominfo(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0, ("Could not retrieve session key: %s\n", nt_errstr(status)));
 		goto done;
@@ -1809,7 +1812,7 @@ static NTSTATUS cmd_lsa_add_priv(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -1918,7 +1921,7 @@ static NTSTATUS cmd_lsa_del_priv(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2019,7 +2022,7 @@ static NTSTATUS cmd_lsa_create_secret(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2079,7 +2082,7 @@ static NTSTATUS cmd_lsa_delete_secret(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2158,7 +2161,7 @@ static NTSTATUS cmd_lsa_query_secret(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2205,7 +2208,8 @@ static NTSTATUS cmd_lsa_query_secret(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
 	}
@@ -2261,7 +2265,7 @@ static NTSTATUS cmd_lsa_set_secret(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2293,7 +2297,8 @@ static NTSTATUS cmd_lsa_set_secret(struct rpc_pipe_client *cli,
 	ZERO_STRUCT(new_val);
 	ZERO_STRUCT(old_val);
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
 	}
@@ -2352,7 +2357,7 @@ static NTSTATUS cmd_lsa_retrieve_private_data(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2382,7 +2387,8 @@ static NTSTATUS cmd_lsa_retrieve_private_data(struct rpc_pipe_client *cli,
 		goto done;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
 	}
@@ -2427,7 +2433,7 @@ static NTSTATUS cmd_lsa_store_private_data(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2444,7 +2450,8 @@ static NTSTATUS cmd_lsa_store_private_data(struct rpc_pipe_client *cli,
 
 	ZERO_STRUCT(val);
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
 	}
@@ -2497,7 +2504,7 @@ static NTSTATUS cmd_lsa_create_trusted_domain(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2580,14 +2587,15 @@ static NTSTATUS cmd_lsa_create_trusted_domain_ex3(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("Could not retrieve session key: %s\n",
 			nt_errstr(status));
 		goto done;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2694,14 +2702,15 @@ static NTSTATUS cmd_lsa_create_trusted_domain_ex2(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = cli_get_session_key(mem_ctx, cli, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("Could not retrieve session key: %s\n",
 			nt_errstr(status));
 		goto done;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
@@ -2789,7 +2798,7 @@ static NTSTATUS cmd_lsa_delete_trusted_domain(struct rpc_pipe_client *cli,
 		return NT_STATUS_OK;
 	}
 
-	status = dcerpc_lsa_open_policy_fallback(b,
+	status = dcerpc_lsa_open_policy_fallback(cli,
 						 mem_ctx,
 						 cli->srv_name_slash,
 						 true,
