@@ -33,4 +33,22 @@ testit "durable_v2_delay.durable_v2_reconnect_delay_msec" $VALGRIND \
 
 rm $delay_inject_conf
 
+error_inject_conf=$(dirname $SMB_CONF_PATH)/error_inject.conf
+
+cat > $error_inject_conf << _EOF
+	kernel share modes = no
+	kernel oplocks = no
+	posix locking = no
+	error_inject:durable_reconnect = st_ex_nlink
+_EOF
+
+testit "durable-v2-regressions.durable_v2_reconnect_bug15624" \
+	$VALGRIND $BINDIR/smbtorture //$SERVER_IP/error_inject \
+	-U$USERNAME%$PASSWORD \
+	--option=torture:bug15624=yes \
+	smb2.durable-v2-regressions.durable_v2_reconnect_bug15624 ||
+	failed=$(expr $failed + 1)
+
+rm $error_inject_conf
+
 testok $0 $failed
