@@ -1724,7 +1724,6 @@ EOF
 	num_create=$(echo "$out_sorted" | grep -c 'create_time:')
 	num_access=$(echo "$out_sorted" | grep -c 'access_time:')
 	num_write=$(echo "$out_sorted" | grep -c 'write_time:')
-	num_change=$(echo "$out_sorted" | grep -c 'change_time:')
 	if [ "$num_create" != "2" ]; then
 		echo "failed - should get two create_time $out"
 		false
@@ -1740,11 +1739,6 @@ EOF
 		false
 		return
 	fi
-	if [ "$num_change" != "1" ]; then
-		echo "failed - should only get one change_time $out"
-		false
-		return
-	fi
 
 	# This could be: Sun Jan  1 05:10:20 AM 2017
 	# or           : Sun Jan  1 05:10:20 2017 CET
