@@ -28,7 +28,6 @@
 #include "passdb/lookup_sid.h"
 #include "auth.h"
 #include "smbprofile.h"
-#include "libsmb/libsmb.h"
 #include "lib/util_ea.h"
 #include "librpc/gen_ndr/ndr_quota.h"
 #include "librpc/gen_ndr/ndr_security.h"
@@ -765,7 +764,7 @@ void reply_ntcreate_and_X(struct smb_request *req)
 	create_timespec = get_create_timespec(conn, fsp, smb_fname);
 	a_timespec = smb_fname->st.st_ex_atime;
 	m_timespec = smb_fname->st.st_ex_mtime;
-	c_timespec = get_change_timespec(conn, fsp, smb_fname);
+	c_timespec = smb_fname->st.st_ex_ctime;
 
 	if (lp_dos_filetime_resolution(SNUM(conn))) {
 		dos_filetime_timespec(&create_timespec);
@@ -1310,7 +1309,7 @@ static void call_nt_transact_create(connection_struct *conn,
 	create_timespec = get_create_timespec(conn, fsp, smb_fname);
 	a_timespec = smb_fname->st.st_ex_atime;
 	m_timespec = smb_fname->st.st_ex_mtime;
-	c_timespec = get_change_timespec(conn, fsp, smb_fname);
+	c_timespec = smb_fname->st.st_ex_ctime;
 
 	if (lp_dos_filetime_resolution(SNUM(conn))) {
 		dos_filetime_timespec(&create_timespec);
