@@ -20,9 +20,9 @@
 #include "includes.h"
 #include "system/filesys.h"
 #include "torture/proto.h"
-#include "libsmb/libsmb.h"
 #include "libsmb/clirap.h"
 #include "util_tdb.h"
+#include "lib/util/string_wrappers.h"
 
 extern int torture_numops;
 
@@ -187,8 +187,7 @@ bool torture_mangle(int dummy)
 		return False;
 	}
 
-	cli_unlink(cli, "\\mangle_test\\*", FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_HIDDEN);
-	cli_rmdir(cli, "\\mangle_test");
+	torture_deltree(cli, "\\mangle_test");
 
 	if (!NT_STATUS_IS_OK(cli_mkdir(cli, "\\mangle_test"))) {
 		printf("ERROR: Failed to make directory\n");
@@ -211,11 +210,7 @@ bool torture_mangle(int dummy)
 		}
 	}
 
-	cli_unlink(cli, "\\mangle_test\\*", FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_HIDDEN);
-	if (!NT_STATUS_IS_OK(cli_rmdir(cli, "\\mangle_test"))) {
-		printf("ERROR: Failed to remove directory\n");
-		return False;
-	}
+	torture_deltree(cli, "\\mangle_test");
 
 	printf("\nTotal collisions %u/%u  - %.2f%%   (%u failures)\n",
 	       collisions, total, (100.0*collisions) / total, failures);
