@@ -20,9 +20,7 @@
 #ifndef _REGEDIT_VALUELIST_H_
 #define _REGEDIT_VALUELIST_H_
 
-#include "includes.h"
 #include <ncurses.h>
-#include <menu.h>
 #include <panel.h>
 
 struct registry_key;
@@ -31,23 +29,44 @@ struct value_item {
 	uint32_t type;
 	DATA_BLOB data;
 	const char *value_name;
-	char *value_desc;
+	char *value;
 	bool unprintable;
 };
 
+struct multilist;
+
 struct value_list {
 	WINDOW *window;
+	WINDOW *sub;
 	PANEL *panel;
-	MENU *menu;
-	ITEM **items;
-	ITEM **empty;
-}
-;
+	size_t nvalues;
+	struct value_item *values;
+	struct multilist *list;
+};
 struct value_list *value_list_new(TALLOC_CTX *ctx, int nlines, int ncols,
 				  int begin_y, int begin_x);
 void value_list_show(struct value_list *vl);
+void value_list_set_selected(struct value_list *vl, bool select);
+const char **value_list_load_names(TALLOC_CTX *ctx, struct registry_key *key);
 WERROR value_list_load(struct value_list *vl, struct registry_key *key);
 void value_list_resize(struct value_list *vl, int nlines, int ncols,
 		       int begin_y, int begin_x);
+struct value_item *value_list_get_current_item(struct value_list *vl);
+void value_list_set_current_item(struct value_list *vl,
+				 const struct value_item *item);
+void value_list_set_current_item_by_name(struct value_list *vl,
+					 const char *name);
+void value_list_driver(struct value_list *vl, int c);
+
+WERROR value_list_load_quick(struct value_list *vl, struct registry_key *key);
+WERROR value_list_sync(struct value_list *vl);
+struct value_item *value_list_find_next_item(struct value_list *vl,
+					     struct value_item *vitem,
+					     const char *s,
+					     regedit_search_match_fn_t match);
+struct value_item *value_list_find_prev_item(struct value_list *vl,
+					     struct value_item *vitem,
+					     const char *s,
+					     regedit_search_match_fn_t match);
 
 #endif
