@@ -85,8 +85,10 @@ static NTSTATUS name_to_ntstatus_check_password(struct auth_method_context *ctx,
 	user_info_dc->sids = talloc(user_info_dc, struct auth_SidAttr);
 	NT_STATUS_HAVE_NO_MEMORY(user_info_dc->sids);
 
-	user_info_dc->sids->sid = global_sid_Anonymous;
-	user_info_dc->sids->attrs = SE_GROUP_DEFAULT_FLAGS;
+	user_info_dc->sids[0] = (struct auth_SidAttr) {
+		.sid = global_sid_Anonymous,
+		.attrs = SE_GROUP_DEFAULT_FLAGS,
+	};
 
 	/* annoying, but the Anonymous really does have a session key, 
 	   and it is all zeros! */
