@@ -31,7 +31,7 @@
 #include "includes.h"
 #include "dsdb/samdb/samdb.h"
 #include "auth/auth.h"
-#include "smbd/service.h"
+#include "samba/service.h"
 #include "lib/messaging/irpc.h"
 #include "param/param.h"
 #include "system/filesys.h"
