@@ -94,8 +94,8 @@ bool dsdb_schema_info_blob_is_valid(const DATA_BLOB *blob)
 /**
  * Parse schemaInfo structure from a data_blob
  * (DATA_BLOB or ldb_val).
- * Suitable for parsing blobs that comes from
- * DRS interface of from LDB database
+ * Suitable for parsing blobs that come from
+ * DRS interface or from LDB database
  */
 WERROR dsdb_schema_info_from_blob(const DATA_BLOB *blob,
 				  TALLOC_CTX *mem_ctx, struct dsdb_schema_info **_schema_info)
@@ -125,7 +125,7 @@ WERROR dsdb_schema_info_from_blob(const DATA_BLOB *blob,
 	schema_info = talloc(mem_ctx, struct dsdb_schema_info);
 	if (!schema_info) {
 		talloc_free(temp_ctx);
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	/* note that we accept revision numbers of zero now - w2k8r2
@@ -167,7 +167,7 @@ WERROR dsdb_blob_from_schema_info(const struct dsdb_schema_info *schema_info,
 /**
  * Compares schemaInfo signatures in dsdb_schema and prefixMap.
  * NOTE: At present function compares schemaInfo values
- * as string without taking into account schemVersion field
+ * as string without taking into account schemaVersion field
  *
  * @return WERR_OK if schemaInfos are equal
  * 	   WERR_DS_DRA_SCHEMA_MISMATCH if schemaInfos are different
@@ -204,6 +204,15 @@ WERROR dsdb_schema_info_cmp(const struct dsdb_schema *schema,
 		return werr;
 	}
 
+	/*
+	 * shouldn't really be possible is dsdb_schema_info_from_blob
+	 * succeeded, this check is just to satisfy static checker
+	 */
+	if (schema_info == NULL) {
+		TALLOC_FREE(frame);
+		return WERR_INVALID_PARAMETER;
+	}
+
 	if (schema->schema_info->revision > schema_info->revision) {
 		/*
 		 * It's ok if our schema is newer than the remote one
