@@ -23,7 +23,7 @@ from samba.credentials import Credentials, DONT_USE_KERBEROS, MUST_USE_KERBEROS,
 from samba.dsdb import GTYPE_SECURITY_GLOBAL_GROUP, GTYPE_SECURITY_UNIVERSAL_GROUP
 import samba.tests
 from samba.tests import delete_force
-from samba.dcerpc import samr, security
+from samba.dcerpc import security
 from samba.auth import AUTH_SESSION_INFO_DEFAULT_GROUPS, AUTH_SESSION_INFO_AUTHENTICATED, AUTH_SESSION_INFO_SIMPLE_PRIVILEGES, AUTH_SESSION_INFO_NTLM
 
 
@@ -87,9 +87,10 @@ class StaticTokenTest(samba.tests.TestCase):
         for s in token.sids:
             self.user_sids.append(str(s))
 
-        # Add asserted identity for Kerberos
+        # Add asserted identity and Claims Valid for Kerberos
         if creds.get_kerberos_state() == MUST_USE_KERBEROS:
             self.user_sids.append(str(security.SID_AUTHENTICATION_AUTHORITY_ASSERTED_IDENTITY))
+            self.user_sids.append(str(security.SID_CLAIMS_VALID))
 
 
     def test_rootDSE_tokenGroups(self):
@@ -142,9 +143,11 @@ class StaticTokenTest(samba.tests.TestCase):
         extra_sids.append(security.SID_WORLD)
         extra_sids.append(security.SID_NT_NETWORK)
         extra_sids.append(security.SID_NT_AUTHENTICATED_USERS)
+        extra_sids.append(security.SID_NT_THIS_ORGANIZATION)
         extra_sids.append(security.SID_BUILTIN_PREW2K)
         if creds.get_kerberos_state() == MUST_USE_KERBEROS:
             extra_sids.append(security.SID_AUTHENTICATION_AUTHORITY_ASSERTED_IDENTITY)
+            extra_sids.append(security.SID_CLAIMS_VALID)
         if creds.get_kerberos_state() == DONT_USE_KERBEROS:
             extra_sids.append(security.SID_NT_NTLM_AUTHENTICATION)
 
@@ -189,7 +192,7 @@ class StaticTokenTest(samba.tests.TestCase):
         server_to_client = b""
 
         # Run the actual call loop.
-        while client_finished == False and server_finished == False:
+        while not client_finished and not server_finished:
             if not client_finished:
                 print("running client gensec_update")
                 (client_finished, client_to_server) = gensec_client.update(server_to_client)
@@ -341,9 +344,10 @@ class DynamicTokenTest(samba.tests.TestCase):
         for s in token.sids:
             self.user_sids.append(str(s))
 
-        # Add asserted identity for Kerberos
+        # Add asserted identity and Claims Valid for Kerberos
         if creds.get_kerberos_state() == MUST_USE_KERBEROS:
             self.user_sids.append(str(security.SID_AUTHENTICATION_AUTHORITY_ASSERTED_IDENTITY))
+            self.user_sids.append(str(security.SID_CLAIMS_VALID))
 
     def tearDown(self):
         super(DynamicTokenTest, self).tearDown()
@@ -414,9 +418,11 @@ class DynamicTokenTest(samba.tests.TestCase):
         extra_sids.append(security.SID_WORLD)
         extra_sids.append(security.SID_NT_NETWORK)
         extra_sids.append(security.SID_NT_AUTHENTICATED_USERS)
+        extra_sids.append(security.SID_NT_THIS_ORGANIZATION)
         extra_sids.append(security.SID_BUILTIN_PREW2K)
         if creds.get_kerberos_state() == MUST_USE_KERBEROS:
             extra_sids.append(security.SID_AUTHENTICATION_AUTHORITY_ASSERTED_IDENTITY)
+            extra_sids.append(security.SID_CLAIMS_VALID)
         if creds.get_kerberos_state() == DONT_USE_KERBEROS:
             extra_sids.append(security.SID_NT_NTLM_AUTHENTICATION)
 
@@ -461,7 +467,7 @@ class DynamicTokenTest(samba.tests.TestCase):
         server_to_client = b""
 
         # Run the actual call loop.
-        while client_finished == False and server_finished == False:
+        while not client_finished and not server_finished:
             if not client_finished:
                 print("running client gensec_update")
                 (client_finished, client_to_server) = gensec_client.update(server_to_client)
@@ -530,7 +536,6 @@ class DynamicTokenTest(samba.tests.TestCase):
         res = self.ldb.search(self.user_sid_dn, scope=ldb.SCOPE_BASE, attrs=["tokenGroups"])
         self.assertEqual(len(res), 1)
 
-        dn_tokengroups = []
         for sid in res[0]['tokenGroups']:
             sid = ndr_unpack(samba.dcerpc.security.dom_sid, sid)
             res3 = self.admin_ldb.search(base="<SID=%s>" % sid, scope=ldb.SCOPE_BASE,
@@ -617,7 +622,6 @@ class DynamicTokenTest(samba.tests.TestCase):
         res = self.ldb.search(self.user_sid_dn, scope=ldb.SCOPE_BASE, attrs=["tokenGroupsGlobalAndUniversal"])
         self.assertEqual(len(res), 1)
 
-        dn_tokengroups = []
         for sid in res[0]['tokenGroupsGlobalAndUniversal']:
             sid = ndr_unpack(samba.dcerpc.security.dom_sid, sid)
             res3 = self.admin_ldb.search(base="<SID=%s>" % sid, scope=ldb.SCOPE_BASE,
@@ -642,16 +646,23 @@ class DynamicTokenTest(samba.tests.TestCase):
                                            domain_sid)
         user_handle = samr_conn.OpenUser(samr_domain, security.SEC_FLAG_MAXIMUM_ALLOWED, user_rid)
         rids = samr_conn.GetGroupsForUser(user_handle)
+        user_info = samr_conn.QueryUserInfo(user_handle, 1)
         samr_dns = set()
+        found_primary_gid = False
         for rid in rids.rids:
-            self.assertEqual(rid.attributes, security.SE_GROUP_MANDATORY | security.SE_GROUP_ENABLED_BY_DEFAULT | security.SE_GROUP_ENABLED)
+            self.assertEqual(rid.attributes, security.SE_GROUP_DEFAULT_FLAGS)
             sid = "%s-%d" % (domain_sid, rid.rid)
             res = self.admin_ldb.search(base="<SID=%s>" % sid, scope=ldb.SCOPE_BASE,
                                         attrs=[])
             samr_dns.add(res[0].dn.get_casefold())
 
-        user_info = samr_conn.QueryUserInfo(user_handle, 1)
-        self.assertEqual(rids.rids[0].rid, user_info.primary_gid)
+            # Note Windows 2025 has the primary_group_rid as
+            # the last element in the rids array
+            if rid.rid == user_info.primary_gid:
+                self.assertFalse(found_primary_gid)
+                found_primary_gid = True
+
+        self.assertTrue(found_primary_gid)
 
         tokenGroupsSet = set()
         res = self.ldb.search(self.user_sid_dn, scope=ldb.SCOPE_BASE, attrs=["tokenGroupsGlobalAndUniversal"])
@@ -670,7 +681,7 @@ class DynamicTokenTest(samba.tests.TestCase):
         memberOf = set()
         # Add the primary group
         primary_group_sid = "%s-%d" % (domain_sid, user_info.primary_gid)
-        res2 = self.admin_ldb.search(base="<SID=%s>" % sid, scope=ldb.SCOPE_BASE,
+        res2 = self.admin_ldb.search(base="<SID=%s>" % primary_group_sid, scope=ldb.SCOPE_BASE,
                                      attrs=[])
 
         memberOf.add(res2[0].dn.get_casefold())
