@@ -166,11 +166,11 @@ static NTSTATUS gp_cli_connect(struct gp_context *gp_ctx)
 	return smbcli_full_connection(gp_ctx,
 			&gp_ctx->cli,
 			gp_ctx->active_dc->name,
-			lpcfg_smb_ports(gp_ctx->lp_ctx),
 			"sysvol",
 			NULL,
 			lpcfg_socket_options(gp_ctx->lp_ctx),
 			gp_ctx->credentials,
+			gp_ctx->lp_ctx,
 			lpcfg_resolve_context(gp_ctx->lp_ctx),
 			gp_ctx->ev_ctx,
 			&options,
