@@ -151,7 +151,10 @@ static void continue_pipe_connect(struct composite_context *ctx)
 	if (s->monitor_fn) {
 		struct monitor_msg msg;
 		struct msg_net_rpc_connect data;
-		const struct dcerpc_binding *b = s->r.out.dcerpc_pipe->binding;
+		struct dcerpc_binding_handle *bh =
+			s->r.out.dcerpc_pipe->binding_handle;
+		const struct dcerpc_binding *b =
+			dcerpc_binding_handle_get_binding(bh);
 
 		/* prepare monitor message and post it */
 		data.host        = dcerpc_binding_get_string_option(b, "host");
@@ -367,7 +370,10 @@ static void continue_rpc_connect(struct composite_context *ctx)
 	if (s->monitor_fn) {
 		struct monitor_msg msg;
 		struct msg_net_rpc_connect data;
-		const struct dcerpc_binding *b = s->r.out.dcerpc_pipe->binding;
+		struct dcerpc_binding_handle *bh =
+			s->r.out.dcerpc_pipe->binding_handle;
+		const struct dcerpc_binding *b =
+			dcerpc_binding_handle_get_binding(bh);
 
 		data.host        = dcerpc_binding_get_string_option(b, "host");
 		data.endpoint    = dcerpc_binding_get_string_option(b, "endpoint");
@@ -550,7 +556,10 @@ static void continue_dci_rpc_connect(struct composite_context *ctx)
 	if (s->monitor_fn) {
 		struct monitor_msg msg;
 		struct msg_net_rpc_connect data;
-		const struct dcerpc_binding *b = s->r.out.dcerpc_pipe->binding;
+		struct dcerpc_binding_handle *bh =
+			s->r.out.dcerpc_pipe->binding_handle;
+		const struct dcerpc_binding *b =
+			dcerpc_binding_handle_get_binding(bh);
 
 		data.host        = dcerpc_binding_get_string_option(b, "host");
 		data.endpoint    = dcerpc_binding_get_string_option(b, "endpoint");
@@ -573,7 +582,8 @@ static void continue_dci_rpc_connect(struct composite_context *ctx)
 
 	s->attr.sec_qos = &s->qos;
 
-	transport = dcerpc_binding_get_transport(s->lsa_pipe->binding);
+	transport = dcerpc_binding_handle_get_transport(
+			s->lsa_pipe->binding_handle);
 	if (transport == NCACN_IP_TCP) {
 		/*
 		 * Skip to creating the actual connection. We can't open a
@@ -784,13 +794,21 @@ static void continue_epm_map_binding_send(struct composite_context *c)
 	struct rpc_connect_dci_state *s;
 	struct composite_context *epm_map_req;
 	struct cli_credentials *epm_creds = NULL;
+	const struct dcerpc_binding *b = NULL;
 
 	s = talloc_get_type(c->private_data, struct rpc_connect_dci_state);
 
 	/* prepare to get endpoint mapping for the requested interface */
-	s->final_binding = dcerpc_binding_dup(s, s->lsa_pipe->binding);
+	b = dcerpc_binding_handle_get_binding(s->lsa_pipe->binding_handle);
+	s->final_binding = dcerpc_binding_dup(s, b);
 	if (composite_nomem(s->final_binding, c)) return;
 
+	/*
+	 * We don't want to inherit the assoc_group_id from the
+	 * lsa_pipe here!
+	 */
+	dcerpc_binding_set_assoc_group_id(s->final_binding, 0);
+
 	epm_creds = cli_credentials_init_anon(s);
 	if (composite_nomem(epm_creds, c)) return;
 
@@ -865,7 +883,10 @@ static void continue_secondary_conn(struct composite_context *ctx)
 	if (s->monitor_fn) {
 		struct monitor_msg msg;
 		struct msg_net_rpc_connect data;
-		const struct dcerpc_binding *b = s->r.out.dcerpc_pipe->binding;
+		struct dcerpc_binding_handle *bh =
+			s->r.out.dcerpc_pipe->binding_handle;
+		const struct dcerpc_binding *b =
+			dcerpc_binding_handle_get_binding(bh);
 
 		/* prepare monitor message and post it */
 		data.host        = dcerpc_binding_get_string_option(b, "host");
