@@ -128,7 +128,7 @@ static struct dnsserver_state *dnsserver_connect(struct dcesrv_call_state *dce_c
 
 	dsstate->samdb = dcesrv_samdb_connect_as_user(dsstate, dce_call);
 	if (dsstate->samdb == NULL) {
-		DEBUG(0,("dnsserver: Failed to open samdb"));
+		DEBUG(0,("dnsserver: Failed to open samdb\n"));
 		goto failed;
 	}
 
@@ -165,7 +165,7 @@ static struct dnsserver_state *dnsserver_connect(struct dcesrv_call_state *dce_c
 				dsstate->zones_count++;
 			} else {
 				/* Ignore duplicate zone */
-				DEBUG(3,("dnsserver: Ignoring duplicate zone '%s' from '%s'",
+				DEBUG(3,("dnsserver: Ignoring duplicate zone '%s' from '%s'\n",
 					 z->name, ldb_dn_get_linearized(z->zone_dn)));
 			}
 			z = znext;
@@ -200,10 +200,10 @@ static WERROR dnsserver_query_server(struct dnsserver_state *dsstate,
 {
 	uint8_t is_integer, is_addresses, is_string, is_wstring, is_stringlist;
 	uint32_t answer_integer;
-	struct IP4_ARRAY *answer_iparray;
-	struct DNS_ADDR_ARRAY *answer_addrarray;
-	char *answer_string;
-	struct DNS_RPC_UTF8_STRING_LIST *answer_stringlist;
+	struct IP4_ARRAY *answer_iparray = NULL;
+	struct DNS_ADDR_ARRAY *answer_addrarray = NULL;
+	char *answer_string = NULL;
+	struct DNS_RPC_UTF8_STRING_LIST *answer_stringlist = NULL;
 	struct dnsserver_serverinfo *serverinfo;
 
 	serverinfo = dsstate->serverinfo;
@@ -729,18 +729,8 @@ static WERROR dnsserver_query_server(struct dnsserver_state *dsstate,
 		}
 		is_addresses = 1;
 	} else if (strcasecmp(operation, "BreakOnReceiveFrom") == 0) {
-		if (client_version == DNS_CLIENT_VERSION_LONGHORN) {
-			answer_addrarray = NULL;
-		} else {
-			answer_iparray = NULL;
-		}
 		is_addresses = 1;
 	} else if (strcasecmp(operation, "BreakOnUpdateFrom") == 0) {
-		if (client_version == DNS_CLIENT_VERSION_LONGHORN) {
-			answer_addrarray = NULL;
-		} else {
-			answer_iparray = NULL;
-		}
 		is_addresses = 1;
 	} else if (strcasecmp(operation, "LogIPFilterList") == 0) {
 		if (client_version == DNS_CLIENT_VERSION_LONGHORN) {
@@ -780,13 +770,10 @@ static WERROR dnsserver_query_server(struct dnsserver_state *dsstate,
 		answer_string = talloc_strdup(mem_ctx, serverinfo->pwszLogFilePath);
 		is_wstring = 1;
 	} else if (strcasecmp(operation, "ServerLevelPluginDll") == 0) {
-		answer_string = NULL;
 		is_wstring = 1;
 	} else if (strcasecmp(operation, "DsBackgroundPauseName") == 0) {
-		answer_string = NULL;
 		is_string = 1;
 	} else if (strcasecmp(operation, "DsNotRoundRobinTypes") == 0) {
-		answer_string = NULL;
 		is_string = 1;
 	}
 
@@ -803,10 +790,8 @@ static WERROR dnsserver_query_server(struct dnsserver_state *dsstate,
 	is_stringlist = 0;
 
 	if (strcasecmp(operation, "GlobalQueryBlockList") == 0) {
-		answer_stringlist = NULL;
 		is_stringlist = 1;
 	} else if (strcasecmp(operation, "SocketPoolExcludedPortRanges") == 0) {
-		answer_stringlist = NULL;
 		is_stringlist = 1;
 	}
 
@@ -816,7 +801,7 @@ static WERROR dnsserver_query_server(struct dnsserver_state *dsstate,
 		return WERR_OK;
 	}
 
-	DEBUG(0,("dnsserver: Invalid server operation %s", operation));
+	DEBUG(0,("dnsserver: Invalid server operation %s\n", operation));
 	return WERR_DNS_ERROR_INVALID_PROPERTY;
 }
 
@@ -831,9 +816,9 @@ static WERROR dnsserver_query_zone(struct dnsserver_state *dsstate,
 {
 	uint8_t is_integer, is_addresses, is_string;
 	uint32_t answer_integer = 0;
-	struct IP4_ARRAY *answer_iparray;
-	struct DNS_ADDR_ARRAY *answer_addrarray;
-	char *answer_string;
+	struct IP4_ARRAY *answer_iparray = NULL;
+	struct DNS_ADDR_ARRAY *answer_addrarray = NULL;
+	char *answer_string = NULL;
 	struct dnsserver_zoneinfo *zoneinfo;
 
 	zoneinfo = z->zoneinfo;
@@ -1021,11 +1006,6 @@ static WERROR dnsserver_query_zone(struct dnsserver_state *dsstate,
 	is_addresses = 0;
 
 	if (strcasecmp(operation, "AllowNSRecordsAutoCreation") == 0) {
-		if (client_version == DNS_CLIENT_VERSION_LONGHORN) {
-			answer_addrarray = NULL;
-		} else {
-			answer_iparray = NULL;
-		}
 		is_addresses = 1;
 	} else if (strcasecmp(operation, "ScavengeServers") == 0) {
 		if (client_version == DNS_CLIENT_VERSION_LONGHORN) {
@@ -1084,7 +1064,6 @@ static WERROR dnsserver_query_zone(struct dnsserver_state *dsstate,
 		answer_string = talloc_strdup(mem_ctx, z->partition->pszDpFqdn);
 		is_string = 1;
 	} else if (strcasecmp(operation, "BreakOnNameUpdate") == 0) {
-		answer_string = NULL;
 		is_string = 1;
 	}
 
@@ -1094,7 +1073,7 @@ static WERROR dnsserver_query_zone(struct dnsserver_state *dsstate,
 		return WERR_OK;
 	}
 
-	DEBUG(0,("dnsserver: Invalid zone operation %s", operation));
+	DEBUG(0,("dnsserver: Invalid zone operation %s\n", operation));
 	return WERR_DNS_ERROR_INVALID_PROPERTY;
 
 }
@@ -1227,11 +1206,11 @@ static WERROR dnsserver_operate_server(struct dnsserver_state *dsstate,
 	}
 
 	if (valid_operation) {
-		DEBUG(0, ("dnsserver: server operation '%s' not implemented", operation));
+		DEBUG(0, ("dnsserver: server operation '%s' not implemented\n", operation));
 		return WERR_CALL_NOT_IMPLEMENTED;
 	}
 
-	DEBUG(0, ("dnsserver: invalid server operation '%s'", operation));
+	DEBUG(0, ("dnsserver: invalid server operation '%s'\n", operation));
 	return WERR_DNS_ERROR_INVALID_PROPERTY;
 }
 
@@ -1510,11 +1489,11 @@ static WERROR dnsserver_complex_operate_server(struct dnsserver_state *dsstate,
 	}
 
 	if (valid_operation) {
-		DEBUG(0, ("dnsserver: server complex operation '%s' not implemented", operation));
+		DEBUG(0, ("dnsserver: server complex operation '%s' not implemented\n", operation));
 		return WERR_CALL_NOT_IMPLEMENTED;
 	}
 
-	DEBUG(0, ("dnsserver: invalid server complex operation '%s'", operation));
+	DEBUG(0, ("dnsserver: invalid server complex operation '%s'\n", operation));
 	return WERR_DNS_ERROR_INVALID_PROPERTY;
 }
 
@@ -1600,11 +1579,11 @@ static WERROR dnsserver_operate_zone(struct dnsserver_state *dsstate,
 	}
 
 	if (valid_operation) {
-		DEBUG(0, ("dnsserver: zone operation '%s' not implemented", operation));
+		DEBUG(0, ("dnsserver: zone operation '%s' not implemented\n", operation));
 		return WERR_CALL_NOT_IMPLEMENTED;
 	}
 
-	DEBUG(0, ("dnsserver: invalid zone operation '%s'", operation));
+	DEBUG(0, ("dnsserver: invalid zone operation '%s'\n", operation));
 	return WERR_DNS_ERROR_INVALID_PROPERTY;
 }
 
@@ -1629,7 +1608,7 @@ static WERROR dnsserver_complex_operate_zone(struct dnsserver_state *dsstate,
 		}
 	}
 
-	DEBUG(0,("dnsserver: Invalid zone operation %s", operation));
+	DEBUG(0,("dnsserver: Invalid zone operation %s\n", operation));
 	return WERR_DNS_ERROR_INVALID_PROPERTY;
 }
 
