@@ -109,7 +109,7 @@ static NTSTATUS test_generate_session_info_pac(struct auth4_context *auth_ctx,
 	talloc_steal(pac_data, pac_data->pac_srv_sig);
 	talloc_steal(pac_data, pac_data->pac_kdc_sig);
 
-	if (user_info_dc->info->authenticated) {
+	if (!(user_info_dc->info->user_flags & NETLOGON_GUEST)) {
 		session_info_flags |= AUTH_SESSION_INFO_AUTHENTICATED;
 	}
 
@@ -147,7 +147,7 @@ static const struct PAC_BUFFER *get_pac_buffer(const struct PAC_DATA *pac_data,
 	return pac_buf;
 }
 
-/* Also happens to be a really good one-step verfication of our Kerberos stack */
+/* Also happens to be a really good one-step verification of our Kerberos stack */
 
 static bool netlogon_validate_pac(struct torture_context *tctx,
 				  struct dcerpc_pipe *p1,
@@ -313,7 +313,7 @@ static bool test_PACVerify(struct torture_context *tctx,
 				       (ndr_pull_flags_fn_t)ndr_pull_PAC_DATA);
 	torture_assert(tctx, NDR_ERR_CODE_IS_SUCCESS(ndr_err), "ndr_pull_struct_blob of PAC_DATA structure failed");
 
-	num_pac_buffers = 6;
+	num_pac_buffers = 7;
 	if (expect_pac_upn_dns_info) {
 		num_pac_buffers += 1;
 	}
@@ -406,6 +406,31 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	struct dcerpc_binding_handle *b = NULL;
 	enum ndr_err_code ndr_err;
 	DATA_BLOB payload, pac_wrapped;
+	enum dcerpc_AuthType auth_type = DCERPC_AUTH_TYPE_NONE;
+	enum dcerpc_AuthLevel auth_level = DCERPC_AUTH_LEVEL_NONE;
+	NTSTATUS status;
+
+	if (negotiate_flags & NETLOGON_NEG_SUPPORTS_KERBEROS_AUTH) {
+		const struct dcerpc_binding *b1 =
+			dcerpc_binding_handle_get_binding(p1->binding_handle);
+		struct dcerpc_binding *b2 = NULL;
+
+		b2 = dcerpc_binding_dup(tctx, b1);
+		torture_assert(tctx, b2 != NULL, "dcerpc_binding_dup");
+		dcerpc_binding_set_flags(b2,
+					 DCERPC_SCHANNEL | DCERPC_SCHANNEL_KRB5,
+					 DCERPC_AUTH_OPTIONS);
+
+		cli_credentials_set_netlogon_creds(server_creds, NULL);
+		status = dcerpc_pipe_connect_b(tctx, &p, b2,
+					       &ndr_table_netlogon,
+					       server_creds,
+					       tctx->ev, tctx->lp_ctx);
+		torture_assert_ntstatus_ok(tctx, status,
+				"dcerpc_pipe_connect_b(DCERPC_SCHANNEL_KRB5)\n");
+		creds = cli_credentials_get_netlogon_creds(server_creds);
+		goto connected;
+	}
 
 	if (!test_SetupCredentials2(p1, tctx, negotiate_flags,
 				    server_creds, secure_channel_type,
@@ -416,6 +441,7 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 				       DCERPC_SIGN | DCERPC_SEAL, &p)) {
 		return false;
 	}
+connected:
 	b = p->binding_handle;
 
 	pac_wrapped_struct.ChecksumLength = pac_data->pac_srv_sig->signature.length;
@@ -437,11 +463,6 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	torture_assert(tctx, NDR_ERR_CODE_IS_SUCCESS(ndr_err), "ndr_push_struct_blob of PACValidate structure failed");
 
 	torture_assert(tctx, (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR), "not willing to even try a PACValidate without RC4 encryption");
-	if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
-		netlogon_creds_aes_encrypt(creds, pac_wrapped.data, pac_wrapped.length);
-	} else {
-		netlogon_creds_arcfour_crypt(creds, pac_wrapped.data, pac_wrapped.length);
-	}
 
 	generic.length = pac_wrapped.length;
 	generic.data = pac_wrapped.data;
@@ -471,6 +492,14 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	r.out.authoritative = &authoritative;
 	r.out.return_authenticator = &return_authenticator;
 
+	dcerpc_binding_handle_auth_info(b, &auth_type, &auth_level);
+	status = netlogon_creds_encrypt_samlogon_logon(creds,
+						       r.in.logon_level,
+						       r.in.logon,
+						       auth_type,
+						       auth_level);
+	torture_assert_ntstatus_ok(tctx, status, "encrypt_samlogon_logon");
+
 	torture_assert_ntstatus_ok(tctx, dcerpc_netr_LogonSamLogon_r(b, tctx, &r),
 		"LogonSamLogon failed");
 
@@ -496,7 +525,11 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 
 	torture_assert_ntstatus_equal(tctx, r.out.result, NT_STATUS_LOGON_FAILURE, "LogonSamLogon failed");
 
-	torture_assert(tctx, netlogon_creds_client_check(creds, &r.out.return_authenticator->cred),
+	status = netlogon_creds_client_verify(creds,
+					      &r.out.return_authenticator->cred,
+					      auth_type,
+					      auth_level);
+	torture_assert_ntstatus_ok(tctx, status,
 		       "Credential chaining failed");
 
 	/* This will break the parsing nicely (even in the crypto wrapping), check we get INVALID_PARAMETER */
@@ -519,8 +552,11 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 
 	torture_assert_ntstatus_equal(tctx, r.out.result, NT_STATUS_INVALID_PARAMETER, "LogonSamLogon failed");
 
-	torture_assert(tctx, netlogon_creds_client_check(creds,
-							 &r.out.return_authenticator->cred),
+	status = netlogon_creds_client_verify(creds,
+					      &r.out.return_authenticator->cred,
+					      auth_type,
+					      auth_level);
+	torture_assert_ntstatus_ok(tctx, status,
 		       "Credential chaining failed");
 
 	pac_wrapped_struct.ChecksumLength = pac_data->pac_srv_sig->signature.length;
@@ -546,11 +582,6 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	torture_assert(tctx, NDR_ERR_CODE_IS_SUCCESS(ndr_err), "ndr_push_struct_blob of PACValidate structure failed");
 
 	torture_assert(tctx, (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR), "not willing to even try a PACValidate without RC4 encryption");
-	if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
-		netlogon_creds_aes_encrypt(creds, pac_wrapped.data, pac_wrapped.length);
-	} else {
-		netlogon_creds_arcfour_crypt(creds, pac_wrapped.data, pac_wrapped.length);
-	}
 
 	generic.length = pac_wrapped.length;
 	generic.data = pac_wrapped.data;
@@ -567,12 +598,24 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	r.in.computer_name = cli_credentials_get_workstation(server_creds);
 	r.in.validation_level = NetlogonValidationGenericInfo2;
 
+	dcerpc_binding_handle_auth_info(b, &auth_type, &auth_level);
+	status = netlogon_creds_encrypt_samlogon_logon(creds,
+						       r.in.logon_level,
+						       r.in.logon,
+						       auth_type,
+						       auth_level);
+	torture_assert_ntstatus_ok(tctx, status, "encrypt_samlogon_logon");
+
 	torture_assert_ntstatus_ok(tctx, dcerpc_netr_LogonSamLogon_r(b, tctx, &r),
 		"LogonSamLogon failed");
 
 	torture_assert_ntstatus_equal(tctx, r.out.result, NT_STATUS_LOGON_FAILURE, "LogonSamLogon failed");
 
-	torture_assert(tctx, netlogon_creds_client_check(creds, &r.out.return_authenticator->cred),
+	status = netlogon_creds_client_verify(creds,
+					      &r.out.return_authenticator->cred,
+					      auth_type,
+					      auth_level);
+	torture_assert_ntstatus_ok(tctx, status,
 		       "Credential chaining failed");
 
 	pac_wrapped_struct.ChecksumLength = pac_data->pac_srv_sig->signature.length;
@@ -598,11 +641,6 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	torture_assert(tctx, NDR_ERR_CODE_IS_SUCCESS(ndr_err), "ndr_push_struct_blob of PACValidate structure failed");
 
 	torture_assert(tctx, (creds->negotiate_flags & NETLOGON_NEG_ARCFOUR), "not willing to even try a PACValidate without RC4 encryption");
-	if (creds->negotiate_flags & NETLOGON_NEG_SUPPORTS_AES) {
-		netlogon_creds_aes_encrypt(creds, pac_wrapped.data, pac_wrapped.length);
-	} else {
-		netlogon_creds_arcfour_crypt(creds, pac_wrapped.data, pac_wrapped.length);
-	}
 
 	generic.length = pac_wrapped.length;
 	generic.data = pac_wrapped.data;
@@ -619,12 +657,24 @@ static bool netlogon_validate_pac(struct torture_context *tctx,
 	r.in.computer_name = cli_credentials_get_workstation(server_creds);
 	r.in.validation_level = NetlogonValidationGenericInfo2;
 
+	dcerpc_binding_handle_auth_info(b, &auth_type, &auth_level);
+	status = netlogon_creds_encrypt_samlogon_logon(creds,
+						       r.in.logon_level,
+						       r.in.logon,
+						       auth_type,
+						       auth_level);
+	torture_assert_ntstatus_ok(tctx, status, "encrypt_samlogon_logon");
+
 	torture_assert_ntstatus_ok(tctx, dcerpc_netr_LogonSamLogon_r(b, tctx, &r),
 		"LogonSamLogon failed");
 
 	torture_assert_ntstatus_equal(tctx, r.out.result, NT_STATUS_INVALID_PARAMETER, "LogonSamLogon failed");
 
-	torture_assert(tctx, netlogon_creds_client_check(creds, &r.out.return_authenticator->cred),
+	status = netlogon_creds_client_verify(creds,
+					      &r.out.return_authenticator->cred,
+					      auth_type,
+					      auth_level);
+	torture_assert_ntstatus_ok(tctx, status,
 		       "Credential chaining failed");
 
 	return true;
@@ -648,6 +698,15 @@ static bool test_PACVerify_bdc_aes(struct torture_context *tctx,
 			      NETLOGON_NEG_AUTH2_ADS_FLAGS | NETLOGON_NEG_SUPPORTS_AES);
 }
 
+static bool test_PACVerify_bdc_krb5(struct torture_context *tctx,
+				    struct dcerpc_pipe *p,
+				    struct cli_credentials *credentials)
+{
+	return test_PACVerify(tctx, p, credentials, SEC_CHAN_BDC,
+			      TEST_MACHINE_NAME_BDC,
+			      NETLOGON_NEG_SUPPORTS_KERBEROS_AUTH);
+}
+
 static bool test_PACVerify_workstation_arcfour(struct torture_context *tctx,
 					       struct dcerpc_pipe *p,
 					       struct cli_credentials *credentials)
@@ -666,6 +725,15 @@ static bool test_PACVerify_workstation_aes(struct torture_context *tctx,
 			      NETLOGON_NEG_AUTH2_ADS_FLAGS | NETLOGON_NEG_SUPPORTS_AES);
 }
 
+static bool test_PACVerify_workstation_krb5(struct torture_context *tctx,
+					    struct dcerpc_pipe *p,
+					    struct cli_credentials *credentials)
+{
+	return test_PACVerify(tctx, p, credentials, SEC_CHAN_WKSTA,
+			      TEST_MACHINE_NAME_WKSTA,
+			      NETLOGON_NEG_SUPPORTS_KERBEROS_AUTH);
+}
+
 #ifdef SAMBA4_USES_HEIMDAL
 static NTSTATUS check_primary_group_in_validation(TALLOC_CTX *mem_ctx,
 						  uint16_t validation_level,
@@ -738,17 +806,25 @@ static bool test_S4U2Self(struct torture_context *tctx,
 	struct auth_session_info *s4u2self_session_info;
 	struct auth_user_info_dc *netlogon_user_info_dc;
 
-	struct netr_NetworkInfo ninfo;
+	struct netr_NetworkInfo ninfo = {};
 	DATA_BLOB names_blob, chal, lm_resp, nt_resp;
 	size_t i;
+	size_t j;
+	size_t k;
 	int flags = CLI_CRED_NTLMv2_AUTH;
 
 	struct dom_sid *builtin_domain;
 
 	struct dom_sid *ai_auth_authority = NULL;
 	struct dom_sid *ai_service = NULL;
+	struct dom_sid *ai_claims_valid = NULL;
 	size_t ai_auth_authority_count = 0;
 	size_t ai_service_count = 0;
+	size_t ai_claims_valid_count = 0;
+	size_t kinit_asserted_identity_index = 0;
+	size_t kinit_claims_valid_index = 0;
+	size_t s4u2self_asserted_identity_index = 0;
+	size_t s4u2self_claims_valid_index = 0;
 	bool ok;
 
 	TALLOC_CTX *tmp_ctx = talloc_new(tctx);
@@ -975,13 +1051,6 @@ static bool test_S4U2Self(struct torture_context *tctx,
 	status = check_primary_group_in_validation(tmp_ctx, r.in.validation_level, r.out.validation);
 	torture_assert_ntstatus_ok(tctx, status, "check_primary_group_in_validation failed");
 
-	/* Check that the primary group is not duplicated in user_info_dc SID array */
-	for (i = 2; i < netlogon_user_info_dc->num_sids; i++) {
-		torture_assert(tctx, !dom_sid_equal(&netlogon_user_info_dc->sids[1],
-						    &netlogon_user_info_dc->sids[i]),
-			       "Duplicate PrimaryGroupId in return SID array");
-	}
-
 	torture_assert_str_equal(tctx, netlogon_user_info_dc->info->account_name == NULL ? "" : netlogon_user_info_dc->info->account_name,
 				 kinit_session_info->info->account_name, "Account name differs for kinit-based PAC");
 	torture_assert_str_equal(tctx,netlogon_user_info_dc->info->account_name == NULL ? "" : netlogon_user_info_dc->info->account_name,
@@ -990,29 +1059,48 @@ static bool test_S4U2Self(struct torture_context *tctx,
 	torture_assert_str_equal(tctx, netlogon_user_info_dc->info->full_name == NULL ? "" : netlogon_user_info_dc->info->full_name, s4u2self_session_info->info->full_name, "Full name differs for S4U2Self");
 
 	builtin_domain = dom_sid_parse_talloc(tmp_ctx, SID_BUILTIN);
+	torture_assert_not_null(tctx, builtin_domain, "failed to parse SID");
 
 	/* KRB5 might have an additional sid, the asserted identity */
 	ai_auth_authority = dom_sid_parse_talloc(
 			tmp_ctx,
 			SID_AUTHENTICATION_AUTHORITY_ASSERTED_IDENTITY);
+	torture_assert_not_null(tctx, ai_auth_authority, "failed to parse SID");
 
 	ai_service = dom_sid_parse_talloc(
 			tmp_ctx,
 			SID_SERVICE_ASSERTED_IDENTITY);
+	torture_assert_not_null(tctx, ai_service, "failed to parse SID");
+
+	/* ...and the Claims Valid SID. */
+	ai_claims_valid = dom_sid_parse_talloc(
+			tmp_ctx,
+			SID_CLAIMS_VALID);
+	torture_assert_not_null(tctx, ai_claims_valid, "failed to parse SID");
 
 	ai_auth_authority_count = 0;
 	ai_service_count = 0;
+	ai_claims_valid_count = 0;
 	for (i = 0; i < kinit_session_info->torture->num_dc_sids; i++) {
-		ok = dom_sid_equal(&kinit_session_info->torture->dc_sids[i],
+		ok = dom_sid_equal(&kinit_session_info->torture->dc_sids[i].sid,
 				   ai_auth_authority);
 		if (ok) {
 			ai_auth_authority_count++;
+			kinit_asserted_identity_index = i;
 		}
 
-		ok = dom_sid_equal(&kinit_session_info->torture->dc_sids[i],
+		ok = dom_sid_equal(&kinit_session_info->torture->dc_sids[i].sid,
 				   ai_service);
 		if (ok) {
 			ai_service_count++;
+			kinit_asserted_identity_index = i;
+		}
+
+		ok = dom_sid_equal(&kinit_session_info->torture->dc_sids[i].sid,
+				   ai_claims_valid);
+		if (ok) {
+			ai_claims_valid_count++;
+			kinit_claims_valid_index = i;
 		}
 	}
 
@@ -1020,20 +1108,32 @@ static bool test_S4U2Self(struct torture_context *tctx,
 		"Kinit authority asserted identity should be (1)");
 	torture_assert_int_equal(tctx, ai_service_count, 0,
 		"Kinit service asserted identity should be (0)");
+	torture_assert_int_equal(tctx, ai_claims_valid_count, 1,
+		"Kinit Claims Valid should be (1)");
 
 	ai_auth_authority_count = 0;
 	ai_service_count = 0;
+	ai_claims_valid_count = 0;
 	for (i = 0; i < s4u2self_session_info->torture->num_dc_sids; i++) {
-		ok = dom_sid_equal(&s4u2self_session_info->torture->dc_sids[i],
+		ok = dom_sid_equal(&s4u2self_session_info->torture->dc_sids[i].sid,
 				   ai_auth_authority);
 		if (ok) {
 			ai_auth_authority_count++;
+			s4u2self_asserted_identity_index = i;
 		}
 
-		ok = dom_sid_equal(&s4u2self_session_info->torture->dc_sids[i],
+		ok = dom_sid_equal(&s4u2self_session_info->torture->dc_sids[i].sid,
 				   ai_service);
 		if (ok) {
 			ai_service_count++;
+			s4u2self_asserted_identity_index = i;
+		}
+
+		ok = dom_sid_equal(&s4u2self_session_info->torture->dc_sids[i].sid,
+				   ai_claims_valid);
+		if (ok) {
+			ai_claims_valid_count++;
+			s4u2self_claims_valid_index = i;
 		}
 	}
 
@@ -1041,16 +1141,33 @@ static bool test_S4U2Self(struct torture_context *tctx,
 		"S4U2Self authority asserted identity should be (0)");
 	torture_assert_int_equal(tctx, ai_service_count, 1,
 		"S4U2Self service asserted identity should be (1)");
+	torture_assert_int_equal(tctx, ai_claims_valid_count, 1,
+		"S4U2Self Claims Valid should be (1)");
 
-	torture_assert_int_equal(tctx, netlogon_user_info_dc->num_sids, kinit_session_info->torture->num_dc_sids - 1, "Different numbers of domain groups for kinit-based PAC");
-	torture_assert_int_equal(tctx, netlogon_user_info_dc->num_sids, s4u2self_session_info->torture->num_dc_sids - 1, "Different numbers of domain groups for S4U2Self");
+	/*
+	 * Subtract 2 to account for the Asserted Identity and Claims Valid
+	 * SIDs.
+	 */
+	torture_assert_int_equal(tctx, netlogon_user_info_dc->num_sids, kinit_session_info->torture->num_dc_sids - 2, "Different numbers of domain groups for kinit-based PAC");
+	torture_assert_int_equal(tctx, netlogon_user_info_dc->num_sids, s4u2self_session_info->torture->num_dc_sids - 2, "Different numbers of domain groups for S4U2Self");
 
-	for (i = 0; i < netlogon_user_info_dc->num_sids; i++) {
-		torture_assert(tctx, dom_sid_equal(&netlogon_user_info_dc->sids[i], &kinit_session_info->torture->dc_sids[i]), "Different domain groups for kinit-based PAC");
-		torture_assert(tctx, dom_sid_equal(&netlogon_user_info_dc->sids[i], &s4u2self_session_info->torture->dc_sids[i]), "Different domain groups for S4U2Self");
-		torture_assert(tctx, !dom_sid_in_domain(builtin_domain, &s4u2self_session_info->torture->dc_sids[i]), "Returned BUILTIN domain in groups for S4U2Self");
-		torture_assert(tctx, !dom_sid_in_domain(builtin_domain, &kinit_session_info->torture->dc_sids[i]), "Returned BUILTIN domain in groups kinit-based PAC");
-		torture_assert(tctx, !dom_sid_in_domain(builtin_domain, &netlogon_user_info_dc->sids[i]), "Returned BUILTIN domian in groups from NETLOGON SamLogon reply");
+	/* Loop over all three SID arrays. */
+	for (i = 0, j = 0, k = 0; i < netlogon_user_info_dc->num_sids; i++, j++, k++) {
+		while (j == kinit_asserted_identity_index || j == kinit_claims_valid_index) {
+			/* Skip over the asserted identity and Claims Valid SIDs. */
+			++j;
+		}
+		while (k == s4u2self_asserted_identity_index || k == s4u2self_claims_valid_index) {
+			/* Skip over the asserted identity and Claims Valid SIDs. */
+			++k;
+		}
+		torture_assert_sid_equal(tctx, &netlogon_user_info_dc->sids[i].sid, &kinit_session_info->torture->dc_sids[j].sid, "Different domain groups for kinit-based PAC");
+		torture_assert_u32_equal(tctx, netlogon_user_info_dc->sids[i].attrs, kinit_session_info->torture->dc_sids[j].attrs, "Different domain group attrs for kinit-based PAC");
+		torture_assert_sid_equal(tctx, &netlogon_user_info_dc->sids[i].sid, &s4u2self_session_info->torture->dc_sids[k].sid, "Different domain groups for S4U2Self");
+		torture_assert_u32_equal(tctx, netlogon_user_info_dc->sids[i].attrs, s4u2self_session_info->torture->dc_sids[k].attrs, "Different domain group attrs for S4U2Self");
+		torture_assert(tctx, !dom_sid_in_domain(builtin_domain, &s4u2self_session_info->torture->dc_sids[k].sid), "Returned BUILTIN domain in groups for S4U2Self");
+		torture_assert(tctx, !dom_sid_in_domain(builtin_domain, &kinit_session_info->torture->dc_sids[j].sid), "Returned BUILTIN domain in groups kinit-based PAC");
+		torture_assert(tctx, !dom_sid_in_domain(builtin_domain, &netlogon_user_info_dc->sids[i].sid), "Returned BUILTIN domain in groups from NETLOGON SamLogon reply");
 	}
 
 	return true;
@@ -1197,7 +1314,7 @@ static bool test_S4U2Proxy(struct torture_context *tctx,
 				       (ndr_pull_flags_fn_t)ndr_pull_PAC_DATA);
 	torture_assert(tctx, NDR_ERR_CODE_IS_SUCCESS(ndr_err), "ndr_pull_struct_blob of PAC_DATA structure failed");
 
-	num_pac_buffers = 8;
+	num_pac_buffers = 9;
 
 	torture_assert_int_equal(tctx, pac_data_struct.version, 0, "version");
 	torture_assert_int_equal(tctx, pac_data_struct.num_buffers, num_pac_buffers, "num_buffers");
@@ -1230,6 +1347,10 @@ static bool test_S4U2Proxy(struct torture_context *tctx,
 	torture_assert_not_null(tctx, pac_buf, "PAC_TYPE_FULL_CHECKSUM");
 	torture_assert_not_null(tctx, pac_buf->info, "PAC_TYPE_FULL_CHECKSUM info");
 
+	pac_buf = get_pac_buffer(&pac_data_struct, PAC_TYPE_CLIENT_CLAIMS_INFO);
+	torture_assert_not_null(tctx, pac_buf, "PAC_TYPE_CLIENT_CLAIMS_INFO");
+	torture_assert_not_null(tctx, pac_buf->info, "PAC_TYPE_CLIENT_CLAIMS_INFO info");
+
 	pac_buf = get_pac_buffer(&pac_data_struct, PAC_TYPE_CONSTRAINED_DELEGATION);
 	torture_assert_not_null(tctx, pac_buf, "PAC_TYPE_CONSTRAINED_DELEGATION");
 	torture_assert_not_null(tctx, pac_buf->info, "PAC_TYPE_CONSTRAINED_DELEGATION info");
@@ -1335,6 +1456,10 @@ struct torture_suite *torture_rpc_remote_pac(TALLOC_CTX *mem_ctx)
 							      &ndr_table_netlogon, TEST_MACHINE_NAME_BDC);
 	torture_rpc_tcase_add_test_creds(tcase, "verify-sig-aes", test_PACVerify_bdc_aes);
 
+	tcase = torture_suite_add_machine_bdc_rpc_iface_tcase(suite, "netr-bdc-krb5",
+							      &ndr_table_netlogon, TEST_MACHINE_NAME_BDC);
+	torture_rpc_tcase_add_test_creds(tcase, "verify-sig-krb5", test_PACVerify_bdc_krb5);
+
 	tcase = torture_suite_add_machine_workstation_rpc_iface_tcase(suite, "netr-mem-arcfour",
 								      &ndr_table_netlogon, TEST_MACHINE_NAME_WKSTA);
 	torture_rpc_tcase_add_test_creds(tcase, "verify-sig-arcfour", test_PACVerify_workstation_arcfour);
@@ -1343,6 +1468,10 @@ struct torture_suite *torture_rpc_remote_pac(TALLOC_CTX *mem_ctx)
 								      &ndr_table_netlogon, TEST_MACHINE_NAME_WKSTA);
 	torture_rpc_tcase_add_test_creds(tcase, "verify-sig-aes", test_PACVerify_workstation_aes);
 
+	tcase = torture_suite_add_machine_workstation_rpc_iface_tcase(suite, "netr-mem-krb5",
+								      &ndr_table_netlogon, TEST_MACHINE_NAME_WKSTA);
+	torture_rpc_tcase_add_test_creds(tcase, "verify-sig-krb5", test_PACVerify_workstation_krb5);
+
 #ifdef SAMBA4_USES_HEIMDAL
 	tcase = torture_suite_add_machine_bdc_rpc_iface_tcase(suite, "netr-bdc-arcfour",
 							      &ndr_table_netlogon, TEST_MACHINE_NAME_S4U2SELF_BDC);
