@@ -22,8 +22,8 @@
 #include "includes.h"
 #include "lib/events/events.h"
 #include "lib/socket/socket.h"
-#include "smbd/service_task.h"
-#include "smbd/service_stream.h"
+#include "samba/service_task.h"
+#include "samba/service_stream.h"
 #include "librpc/gen_ndr/winsrepl.h"
 #include "wrepl_server/wrepl_server.h"
 #include "nbt_server/wins/winsdb.h"
